#include "xbasic_types.h"
#include "xparameters.h"
#include "xuartns550.h"
#include "xintc.h"
#include "xgpio.h"

#include "usbio.h"
#include "midiin.h"
#include "mididec.h"
#include "usbdec.h"
#include "synth.h"
#include "midi.h"

#include "mb_interface.h"

static XIntc xps_intc_0;
static XUartNs550 MIDI_UART;
static XUartNs550 USB_UART;

int main (void) {
	/* Initialize Interrupt Controller */
	XIntc_Initialize(&xps_intc_0, XPAR_INTC_0_DEVICE_ID);

	/* Register interrupt handlers with intc */
	XIntc_Connect(&xps_intc_0, XPAR_XPS_INTC_0_MIDI_UART_IP2INTC_IRPT_INTR, (XInterruptHandler) XUartNs550_InterruptHandler, &MIDI_UART);
	XIntc_Connect(&xps_intc_0, XPAR_XPS_INTC_0_USB_UART_IP2INTC_IRPT_INTR, (XInterruptHandler) XUartNs550_InterruptHandler, &USB_UART);
	
	/* Start intc */
	XIntc_Start(&xps_intc_0, XIN_REAL_MODE);
	
	/* Enable intc interrupts */
	XIntc_Enable(&xps_intc_0, XPAR_XPS_INTC_0_MIDI_UART_IP2INTC_IRPT_INTR);
	XIntc_Enable(&xps_intc_0, XPAR_XPS_INTC_0_USB_UART_IP2INTC_IRPT_INTR);

	/* Register the interrupt controller handler with the MB Interrupt table */
	microblaze_register_handler((XInterruptHandler) XIntc_InterruptHandler, &xps_intc_0);
	
	/* Enable interrupts in MB */
	microblaze_enable_interrupts();
	
	/* Setup & Initialize MIDI_UART & USB_UART */
	midiin_init(&MIDI_UART, XPAR_MIDI_UART_DEVICE_ID);
	usbio_init(&USB_UART, XPAR_USB_UART_DEVICE_ID);

		
	
	// will also initialize one synthesizer that occupies all 16 channels and listens to midi channel 0
	// To change number of synthesizers and their corresponding polyphony, sysexes must be sent
	midi_init();

	while(1) {
		if (MidiReadItr != MidiWriteItr)
			decodeMidi();
		
		if (usbInReadItr != usbInWriteItr)
			decodeUsb();

		/* not used
		if (outgoingUsb)
			sendUSB(&USB_UART);
		*/
	}
	return 0;
}
