#include "hardware.h"
#include "mb_interface.h"
#include "xparameters.h"

#include "xbasic_types.h"
#include "xparameters.h"
#include "xuartns550.h"
#include "xintc.h"
#include "xgpio.h"
#include "mb_interface.h"

// this call will also "compress" the 32-bit rate to the 22-bit representation
// used inside EGS. 0 is the encoding for immediate.
void hw_send_op_rate(u8 ch, u8 op, u8 stage, u32 value)
{
	u32 fsl_value;
	u8 i;
	rate_t op_rate;	
	op_rate.ch = ch;
	op_rate.op = op;
	op_rate.type = RATE_PARAM;
	op_rate.stage = stage;
	if(!(value))
	{
		op_rate.shift = 0xF;
		op_rate.value = 0x3FFFF;
	} else { // apply shift compression. 32-bit value is converted into
	// a 22-bit float with 4-bit exponent (left shift amount) and 18-bit
	// mantissa (no given 1)
		i = 31;
		while((((1 << i) & value) == 0) && i != 0)
			--i; // i is the index of highest bit set
		if(i > 17) // all bits don't fit (bit 0 is also bit)
		{
			op_rate.shift = (i-17);
			op_rate.value = (value >> (i-17));
		} else {
			op_rate.shift = 0;
			op_rate.value = value;
		}
	}
	fsl_value = (*(u32 *)(&op_rate));
	putfsl(fsl_value, 0);
}

void hw_send_op_eglvl(u8 ch, u8 op, u8 stage, u16 value)
{
	param_t op_other;
	u32 fsl_value;
	op_other.ch = ch;
	op_other.op = op;
	op_other.type = OTHER_PARAM;
	op_other.param = stage & 0x3;
	op_other.value = value;
	fsl_value = (*(u32 *)(&op_other));
	putfsl(fsl_value, 0);		
}

void hw_send_op_freq(u8 ch, u8 op, u32 value)
{
	param_t op_other;
	u32 fsl_value;
	op_other.ch = ch;
	op_other.op = op;
	op_other.type = OTHER_PARAM;
	op_other.param = BASE_FREQ;
	op_other.value = value;
	fsl_value = (*(u32 *)(&op_other));
	putfsl(fsl_value, 0);		
}	

void hw_send_op_amodsense(u8 ch, u8 op, u16 value)
{
	param_t op_other;
	u32 fsl_value;
	op_other.ch = ch;
	op_other.op = op;
	op_other.type = OTHER_PARAM;
	op_other.param = AMP_MOD_SENS;
	op_other.value = value;
	fsl_value = (*(u32 *)(&op_other));
	putfsl(fsl_value, 0);	
}
void hw_send_op_levelpars(u8 ch, u8 op, u16 outlevel, u8 sync, u8 op_on)
{
	param_t op_other;
	op_grp_t values;
	u32 fsl_value;
	op_other.ch = ch;
	op_other.op = op;
	op_other.type = OTHER_PARAM;
	op_other.param = BASE_AMP__SYNC_ON__OPERATOR_ON;
	values.pad = 0; // May be unneccessary to set
	values.base_amp = outlevel;
	values.sync_on = sync;
	values.op_on = op_on;	
	op_other.value = *((u32*)(&values));
	fsl_value = (*(u32 *)(&op_other));
	putfsl(fsl_value, 0);	
}

void hw_send_peg_rate(u8 ch, u8 stage, u32 value)
{
	hw_send_op_rate(ch, GLOBAL, stage, value);
}

void hw_send_peg_lvl(u8 ch, u8 stage, u16 value)
{
	hw_send_op_eglvl(ch, GLOBAL, stage, value);
}

void hw_send_lfo_delay(u8 ch, u32 value)
{
	hw_send_op_freq(ch, GLOBAL, value);
}

void hw_send_pms(u8 ch, u32 value)
{
	hw_send_op_amodsense(ch, GLOBAL, value);
}

void hw_send_others(u8 ch, u16 lfo_speed, u8 lfo_sync, u8 lfo_wave, u8 key_on)
{
	param_t op_other;
	gl_grp_t glpars;
	u32 fsl_value;
	op_other.ch = ch;
	op_other.op = GLOBAL;
	op_other.type = OTHER_PARAM;
	op_other.param = LFO_SPEED__LFO_SYNC__LFO_WAVE__KEY_ON;
	glpars.lfo_speed = lfo_speed;
	glpars.lfo_sync = lfo_sync;
	glpars.lfo_wave = lfo_wave;
	glpars.key_on = key_on;
	op_other.value = *((u32*)(&glpars));
	fsl_value = (*(u32 *)(&op_other));
	putfsl(fsl_value, 0);	
}

void hw_send_algfb(u8 ch, u8 algorithm, u8 feedback)
{
	op_alg_t algfb;
	u32 fsl_value;
	algfb.ch = ch;
	algfb.op = NON_EGS;
	algfb.comp = ALGORITHM; 
	algfb.alg = algorithm;
	algfb.fb_lvl = feedback;
	fsl_value = (*(u32 *)(&algfb));
	putfsl(fsl_value, 0);
}

void hw_send_mixer(u8 type, u8 value)
{
	u32 fsl_value;
	mixer_param_t mixpar;
	mixpar.op = NON_EGS;
	mixpar.comp = type;
	mixpar.value = value;
	fsl_value = (*(u32 *)(&mixpar));
	putfsl(fsl_value, 0);
}
