#ifndef HARDWARE_H
#define HARDWARE_H

//required for bit fields
#include "mb_interface.h"

#define TOTAL_CH (16)
#define TOTAL_OP (6)
#define TOTAL_STAGE (4)
#define TOTAL_PARAM (7)

#define MAX_RATE (4194303) /* (2^22)-1 */
#define MAX_LVL (4095) /* (2^21)-1 */
#define MIN_LVL (0)

#define OCTAVE_SIZE (12)

typedef struct op_grp {
	u32 pad : 18;
	u32 base_amp : 12;
	u32 sync_on : 1;
	u32 op_on :1;
} op_grp_t;

typedef union op_param {
	u32 zero_value : 32;
	u32 lvl_1 : 12;
	u32 lvl_2 : 12;
	u32 lvl_3 : 12;
	u32 lvl_4 : 12;
	u32 base_freq : 20;
	u32 amp_sens : 12;
	u32 op_grp_value : 14;
} op_param_t;

typedef struct gl_grp {
	u32 pad : 15;
	u32 lfo_speed : 12;
	u32 lfo_sync : 1;
	u32 lfo_wave : 3;
	u32 key_on : 1;
} gl_grp_t;

typedef union gl_param {
	u32 zero_value : 32;
	u32 lvl_1 : 12;
	u32 lvl_2 : 12;
	u32 lvl_3 : 12;
	u32 lvl_4 : 12;
	u32 lfo_delay : 20;
	u32 pitch_sens : 12;
	u32 gl_grp_value : 17;
} gl_param_t;

typedef struct rate {
	u32 ch : 4;
	u32 op : 3;
	u32 type : 1;
	u32 stage : 2;
	u32 shift : 4;
	u32 value : 18;	
} rate_t;

typedef struct param {
	u32 ch : 4;
	u32 op : 3;
	u32 type : 1;
	u32 param : 3;
	u32 value : 21; /* Value is right adjusted */
} param_t;

typedef struct op_alg {
	u32 ch : 4;
	u32 op : 3;
	u32 comp : 2;
	u32 pad : 15;
	u32 alg : 5;
	u32 fb_lvl : 3;
} op_alg_t;

typedef struct mixer_param {
	u32 ch : 4;
	u32 op : 3;
	u32 comp : 2;
	u32 pad : 15;
	u32 value : 8;
} mixer_param_t;

typedef union fsl_msg {
	rate_t rate_msg;
	param_t param_msg;
	op_alg_t ops_mdg;
	mixer_param_t mixer_msg;
} fsl_msg_t;

enum op_np {
	OP6 = 0,
	OP5, 
	OP4,
	OP3,
	OP2,
	OP1,
	GLOBAL,
	NON_EGS
};

enum param_type {
	OTHER_PARAM = 0,
	RATE_PARAM
};

enum rate_stage {
	RATE1 = 0,
	RATE2,
	RATE3,
	RATE4
};

enum op_parameter {
	EG_LVL1 = 0,
	EG_LVL2,
	EG_LVL3,
	EG_LVL4,
	BASE_FREQ,
	AMP_MOD_SENS,
	BASE_AMP__SYNC_ON__OPERATOR_ON
};

enum gl_parameter {
	PEG_LVL1 = 0,
	PEG_LVL2,
	PEG_LVL3,
	PEG_LVL4,
	LFO_DELAY,
	PITCH_MOD_SENSE,
	LFO_SPEED__LFO_SYNC__LFO_WAVE__KEY_ON = 7
};

enum component {
	ALGORITHM = 1,
	VOLUME,
	PAN
};

void hw_send_op_rate(u8 ch, u8 op, u8 stage, u32 value); // this call will "compress" the rate as well..
void hw_send_op_eglvl(u8 ch, u8 op, u8 stage, u16 value);
void hw_send_op_freq(u8 ch, u8 op, u32 value);
void hw_send_op_amodsense(u8 ch, u8 op, u16 value);
void hw_send_op_levelpars(u8 ch, u8 op, u16 outlevel, u8 sync, u8 op_on);

void hw_send_peg_rate(u8 ch, u8 stage, u32 value);
void hw_send_peg_lvl(u8 ch, u8 stage, u16 value);
void hw_send_lfo_delay(u8 ch, u32 value);
void hw_send_pms(u8 ch, u32 value);
void hw_send_others(u8 ch, u16 lfo_speed, u8 lfo_sync, u8 lfo_wave, u8 key_on);

void hw_send_algfb(u8 ch, u8 algorithm, u8 feedback);
void hw_send_mixer(u8 type, u8 value);

#endif
