#include "midi.h"
#include "synth.h"
#include "xparameters.h"
#include "dx7_converter.h"

synth_t* synth_tbl[TOTAL_MIDI_CH];
micron_ram_t* micron_ram;

u8* SysexRcvBuffer;
u16 SysexRcvIndex;

u8 CurPatchNbr;

synth_t* midi_getusbdevice()
{
	return synth_tbl[0];
}


void midi_createsynth(u8 channel, u8 polyphony)
{
	synth_init(&(micron_ram->synthesizers[channel]), polyphony,0);
	synth_tbl[channel] = &(micron_ram->synthesizers[channel]);
}

void midi_init()
{
	int i;
	micron_ram = (micron_ram_t*)((u8 *)XPAR_MICRON_RAM_MEM0_BASEADDR); // micron ram area
	SysexRcvBuffer = (u8*)(&(micron_ram->SysexRcvBuffer));
	SysexRcvIndex = 0;
	for(i = 0; i < TOTAL_MIDI_CH; ++i)
		synth_tbl[i] = 0; //initialize synthesizer table to NULL att all channels
	
	//Creating 1 synth which use all channels and operates at midi ch 0
	midi_createsynth(0,16);
}

void midi_noteoff(u8 channel, u8 notevalue, u8 velocity){
	synth_noteoff(synth_tbl[channel], notevalue, velocity);
}

void midi_noteon(u8 channel, u8 notevalue, u8 velocity){
	synth_noteon(synth_tbl[channel], notevalue, velocity);
}

void midi_cc(u8 channel, u8 parameter, u8 value){
	synth_cc(synth_tbl[channel], parameter, value);
}

void midi_pc(u8 channel, u8 patch_nbr){
	synth_t* synth_handle = synth_tbl[channel];
	synth_handle->patch_no = patch_nbr; // unused for now, but could be useful
	//when sysex bank receive is implemented properly.
	CurPatchNbr = patch_nbr;
	load_dx7patch(synth_tbl[channel], &(micron_ram->patches[patch_nbr]));
	synth_refresh_hw(synth_handle);
}

void midi_pw(u8 channel, u16 value){
	synth_pw(synth_tbl[channel], value);
}

void midi_rcvsysexbyte(u8 value){
	SysexRcvBuffer[SysexRcvIndex++] = value;
}

void midi_decodesysex(){
	u8 i;
	//Make room for a new sysex message
	SysexRcvIndex = 0;
	
	//if a bank is received, store it in the range we're using
	//eg. if 40 is selected, store bank in 32-63
	
	//check type, may add additional types for control packets and forward to 
	//synth_decodesysex()... For now, we ignore all non-yamaha packets
	if(SysexRcvBuffer[SYSEX_VENDOR_OFFSET] != SYSEX_VENDOR_YAMAHA_ID)
		return;
	
	if(SysexRcvBuffer[SYSEX_SUBSTATUS_OFFSET] == SYSEX_BULKDATA_TYPE)
	{
		if(SysexRcvBuffer[SYSEX_VOICE_ID_OFFSET] == SYSEX_1_VOICE_ID){	
			u8 sysex_idx = SYSEX_VOICE_DATA_OFFSET;				
			dx7_patch_t* voice_patch = &(micron_ram->patches[CurPatchNbr]);		
			u8 i = 0, j = 0, k = 0;		
			for(i = 0; i < TOTAL_OP; ++i){
				u8* oper = (u8*)(&(voice_patch->operators[i]));
				for(j = 0; j < SYSEX_OP_SIZE; ++j){
					oper[j] = SysexRcvBuffer[sysex_idx++];
				}
			}		
			u8* glob = (u8*)(&(voice_patch->globals));
			for(k = 0; k < SYSEX_GL_SIZE; ++k){
				glob[k] = SysexRcvBuffer[sysex_idx++];
			}
			// reload current patch to get the new one.
			synth_t* synth_handle = synth_tbl[0];
			load_dx7patch(synth_handle, voice_patch);
			synth_refresh_hw(synth_handle);
		}
		else if(SysexRcvBuffer[SYSEX_VOICE_ID_OFFSET] == SYSEX_32_VOICE_ID){	
			// Bank dump. Not working and we need to save space, hence uncommented
			/*
			xil_printf("received bank dump!\r\n");
			u8 sysex_idx = SYSEX_VOICE_DATA_OFFSET;		
			u8 bank_no = (CurPatchNbr/32)*32;
			dx7_patch_t* voice_patch_bank = &(micron_ram->patches[bank_no]);
			u8 bank_idx;		
			for(bank_idx = 0; bank_idx < BANK_SIZE; ++bank_idx){
				dx7_patch_t* voice_patch = &(voice_patch_bank[bank_idx]);
							
				u8 i = 0;		
				for(i = 0; i < TOTAL_OP; ++i){			
					dx7_op_t* oper = &(voice_patch->operators[i]);
					oper->rate[0] = SysexRcvBuffer[sysex_idx++];
					oper->rate[1] = SysexRcvBuffer[sysex_idx++];
					oper->rate[2] = SysexRcvBuffer[sysex_idx++];
					oper->rate[3] = SysexRcvBuffer[sysex_idx++];
					oper->lvl[0] = SysexRcvBuffer[sysex_idx++];
					oper->lvl[1] = SysexRcvBuffer[sysex_idx++];
					oper->lvl[2] = SysexRcvBuffer[sysex_idx++];
					oper->lvl[3] = SysexRcvBuffer[sysex_idx++];
					oper->scal_brk_point = SysexRcvBuffer[sysex_idx++];				
					oper->scal_l_depth = SysexRcvBuffer[sysex_idx++];
					oper->scal_r_depth = SysexRcvBuffer[sysex_idx++];					
					oper->scal_r_curve = (SysexRcvBuffer[sysex_idx] & RIGHT_CURVE_MASK) >> RIGHT_CURVE_SHIFT;
					oper->scal_l_curve = (SysexRcvBuffer[sysex_idx++] & LEFT_CURVE_MASK);							
					oper->osc_detune = (SysexRcvBuffer[sysex_idx] & DETUNE_MASK) >> DETUNE_SHIFT;
					oper->rate_scal = (SysexRcvBuffer[sysex_idx++] & RATE_SCALE_MASK);							
					oper->vel_sense = (SysexRcvBuffer[sysex_idx] & KVS_MASK) >> KVS_SHIFT;
					oper->ams = (SysexRcvBuffer[sysex_idx++] & AMS_MASK);				
					oper->outlvl = SysexRcvBuffer[sysex_idx++];				
					oper->osc_coarse = (SysexRcvBuffer[sysex_idx] & COARSE_MASK) >> COARSE_SHIFT;
					oper->osc_mode = (SysexRcvBuffer[sysex_idx++] & MODE_MASK);				
					oper->osc_fine = SysexRcvBuffer[sysex_idx++];				
				}
		
				dx7_global_t* glob = &(voice_patch->globals);
				glob->peg_rate[0] = SysexRcvBuffer[sysex_idx++];
				glob->peg_rate[1] = SysexRcvBuffer[sysex_idx++];
				glob->peg_rate[2] = SysexRcvBuffer[sysex_idx++];
				glob->peg_rate[3] = SysexRcvBuffer[sysex_idx++];
				glob->peg_lvl[0] = SysexRcvBuffer[sysex_idx++];
				glob->peg_lvl[1] = SysexRcvBuffer[sysex_idx++];
				glob->peg_lvl[2] = SysexRcvBuffer[sysex_idx++];
				glob->peg_lvl[3] = SysexRcvBuffer[sysex_idx++];
				glob->alg = (SysexRcvBuffer[sysex_idx++] & ALG_MASK);			
				glob->osc_sync = (SysexRcvBuffer[sysex_idx] & KEY_SYNC_MASK) >> KEY_SYNC_SHIFT;
				glob->feedbk = (SysexRcvBuffer[sysex_idx++] & FB_MASK);			
				glob->lfo_speed = SysexRcvBuffer[sysex_idx++];
				glob->lfo_delay = SysexRcvBuffer[sysex_idx++];
				glob->pmod_depth = SysexRcvBuffer[sysex_idx++];
				glob->amod_depth = SysexRcvBuffer[sysex_idx++];			
				glob->pmod_sense = (SysexRcvBuffer[sysex_idx] & PT_MOD_SENSE_MASK) >> PT_MOD_SENSE_SHIFT;
				glob->lfo_wave = (SysexRcvBuffer[sysex_idx] & LFO_WAVE_MASK) >> LFO_WAVE_SHIFT;
				glob->lfo_sync = (SysexRcvBuffer[sysex_idx++] & LFO_SYNC_MASK);			
				glob->transpose = SysexRcvBuffer[sysex_idx++];
				glob->name1 = SysexRcvBuffer[sysex_idx++];
				glob->name2 = SysexRcvBuffer[sysex_idx++];
				glob->name3 = SysexRcvBuffer[sysex_idx++];
				glob->name4 = SysexRcvBuffer[sysex_idx++];
				glob->name5 = SysexRcvBuffer[sysex_idx++];
				glob->name6 = SysexRcvBuffer[sysex_idx++];
				glob->name7 = SysexRcvBuffer[sysex_idx++];
				glob->name8 = SysexRcvBuffer[sysex_idx++];
				glob->name9 = SysexRcvBuffer[sysex_idx++];
				glob->name10 = SysexRcvBuffer[sysex_idx++];			
			}
			midi_pc(0, CurPatchNbr); // reload current patch*/
		} else{
			// Do nothing
		}
	} else if(SysexRcvBuffer[SYSEX_SUBSTATUS_OFFSET] == SYSEX_PARDATA_TYPE) {
		//TODO:decode parameter data if time permits
	}
}
