#ifndef MIDI_H 
#define MIDI_H

#include "mb_interface.h"
#include "synth.h"

#define TOTAL_MIDI_CH (16)
#define SYSEX_RCV_BUF_SIZE (8192)
#define MAX_PATCHES (128)

#define SYSEX_VENDOR_OFFSET (0)
#define SYSEX_VENDOR_YAMAHA_ID (0x43)
#define SYSEX_SUBSTATUS_OFFSET (1)
#define SYSEX_BULKDATA_TYPE (0)
#define SYSEX_PARDATA_TYPE (0x10)
#define SYSEX_VOICE_ID_OFFSET (2)
#define SYSEX_1_VOICE_ID (0)
#define SYSEX_32_VOICE_ID (9)
#define SYSEX_VOICE_DATA_OFFSET (5)
#define TOTAL_OP (6)
#define SYSEX_OP_SIZE (21)
#define SYSEX_GL_SIZE (29)
#define BANK_SIZE (32)

#define RIGHT_CURVE_MASK 0x0C // (0000 1100)
#define RIGHT_CURVE_SHIFT 2
#define LEFT_CURVE_MASK 0x03 // (0000 0011)
#define DETUNE_MASK 0x78 // (0111 1000)
#define DETUNE_SHIFT 3
#define RATE_SCALE_MASK	0x07 // (0000 0111)
#define KVS_MASK 0x1C // (0001 1100)
#define KVS_SHIFT 2
#define AMS_MASK 0x03 // (0000 0011)
#define COARSE_MASK 0x3E // (0011 1110)
#define COARSE_SHIFT 1
#define MODE_MASK 0x01 // (0000 0001)
#define ALG_MASK 0x1F // (0001 1111)			
#define KEY_SYNC_MASK 0x08 // (0000 1000)
#define KEY_SYNC_SHIFT 3			
#define FB_MASK 0x07 // (0000 0111)			
#define PT_MOD_SENSE_MASK 0x70 // (0111 0000)
#define PT_MOD_SENSE_SHIFT 4			
#define LFO_WAVE_MASK 0x0E // (0000 1110)
#define LFO_WAVE_SHIFT 1			
#define LFO_SYNC_MASK 0x01 // (0000 0001)

synth_t* midi_getusbdevice();
void midi_createsynth(u8 channel, u8 polyphony);
void midi_init();
void midi_noteoff(u8 channel, u8 notevalue, u8 velocity);
void midi_noteon(u8 channel, u8 notevalue, u8 velocity);
void midi_cc(u8 channel, u8 parameter, u8 value);
void midi_pc(u8 channel, u8 patch_nbr);
void midi_pw(u8 channel, u16 value);
void midi_rcvsysexbyte(u8 value);
void midi_decodesysex();

typedef struct dx7_op {
	u8 rate[4];
	u8 lvl[4];
	u8 scal_brk_point;
	u8 scal_l_curve;
	u8 scal_l_depth;
	u8 scal_r_curve;
	u8 scal_r_depth;
	u8 rate_scal;
	u8 ams;
	u8 vel_sense;
	u8 outlvl;
	u8 osc_mode;
	u8 osc_coarse;
	u8 osc_fine;
	u8 osc_detune;
} dx7_op_t;

typedef struct dx7_global {
	u8 peg_rate[4];
	u8 peg_lvl[4];
	u8 alg;
	u8 feedbk;
	u8 osc_sync;
	u8 lfo_speed;
	u8 lfo_delay;
	u8 pmod_depth;
	u8 amod_depth;
	u8 lfo_sync;
	u8 lfo_wave;
	u8 pmod_sense;
	u8 transpose;
	u8 name1;
	u8 name2;
	u8 name3;
	u8 name4;
	u8 name5;
	u8 name6;
	u8 name7;
	u8 name8;
	u8 name9;
	u8 name10;
} dx7_global_t;

typedef struct dx7_patch {
	dx7_op_t			operators[6];
	dx7_global_t	globals;
} dx7_patch_t;

typedef struct micron_ram {
	u8					SysexRcvBuffer[SYSEX_RCV_BUF_SIZE];
	synth_t			synthesizers[TOTAL_MIDI_CH];
	dx7_patch_t		patches[MAX_PATCHES];
	u8					RecvMidiBuffer[256];
} micron_ram_t;



#endif 
