#include "mididec.h"

void decodeMidi() {
	static u8 SysexMode = 0;
	u8 recvByte;
	recvByte = RecvMidiBuffer[MidiReadItr];
	
	if(recvByte & MIDI_TYPE_MASK) { // Check whether it is a status byte.
		u8 status = (recvByte & MIDI_STATUS_MASK);
		u8 channel = (recvByte & MIDI_CHANNEL_MASK);
		u8 specialtype = channel;
		u8 bytesAvail = MidiWriteItr - MidiReadItr;
		
		switch(status)
		{
			u8 notevalue, velocity;
			u8 parameter, value;
			u8 patch_nbr;
			u8 lsb, msb;
			case NOTE_OFF:
				if(bytesAvail < 3)
					return;
				MidiReadItr++; // status byte read
				notevalue = RecvMidiBuffer[MidiReadItr++];
				velocity = RecvMidiBuffer[MidiReadItr];
				if((notevalue & MIDI_TYPE_MASK) || (velocity & MIDI_TYPE_MASK))
				{
					MidiReadItr--; //Midi Data Error
				} else {
					midi_noteoff(channel, notevalue, velocity);
					MidiReadItr++;
				}
				break;
			case NOTE_ON:
				if(bytesAvail < 3)
					return;
				MidiReadItr++; // status byte read
				notevalue = RecvMidiBuffer[MidiReadItr++];
				velocity = RecvMidiBuffer[MidiReadItr];
				if((notevalue & MIDI_TYPE_MASK) || (velocity & MIDI_TYPE_MASK))
				{
					MidiReadItr--; //Midi Data Error
				} else {
					midi_noteon(channel, notevalue, velocity);
					MidiReadItr++;
				}
				break;
			case POLY_AT: // Discard
				MidiReadItr++;
				break;
			case CC:
				if(bytesAvail < 3)
					return;
				MidiReadItr++;
				parameter = RecvMidiBuffer[MidiReadItr++];
				value = RecvMidiBuffer[MidiReadItr];
				if((parameter & MIDI_TYPE_MASK) || (value & MIDI_TYPE_MASK))
				{
					MidiReadItr--; //Midi Data Error
				} else {
					midi_cc(channel, parameter, value);
					MidiReadItr++;
				}
				break;
			case PC:
				if(bytesAvail < 2 )
					return;
				MidiReadItr++;
				patch_nbr = RecvMidiBuffer[MidiReadItr];
				if(!(patch_nbr & MIDI_TYPE_MASK))
				{
					midi_pc(channel, patch_nbr);
					MidiReadItr++;
				}
				break;
			case AT: // Discard
				MidiReadItr++;
				break;
			case PW:
				if(bytesAvail < 3)
					return;
				MidiReadItr++;
				lsb = RecvMidiBuffer[MidiReadItr++];
				msb = RecvMidiBuffer[MidiReadItr];
				if( (lsb & MIDI_TYPE_MASK) || (msb & MIDI_TYPE_MASK) )
				{
					MidiReadItr--;
				} else {
					u16 val = (msb << 7) | lsb; // Midi has 7-bit data
					midi_pw(channel, val);
					MidiReadItr++;
				}
				break;
			case SPECIAL:
				MidiReadItr++;
				switch(specialtype)
				{
					case SYSEX_BEGIN: // set sysexmode
						SysexMode = 1; //on
						return;
					case SYSEX_END: //decode sysex (default)
						midi_decodesysex();
						break;
				}
				
		}
		
		SysexMode = 0; //Unset sysexmode per default
	} else if(SysexMode) {
		midi_rcvsysexbyte(RecvMidiBuffer[MidiReadItr++]); // copy to SysexBuffer
	} else { // Discard data
		MidiReadItr++;
	}
}
