#ifndef SYNTH_H
#define SYNTH_H

#include "mb_interface.h"

#define TRANSPOSE_MID (24)

enum lfo_wave {
	LFO_TRIANGLE = 0,
	LFO_SAW_DOWN ,
	LFO_SAW_UP ,
	LFO_SQUARE ,
	LFO_SINE ,
	LFO_SH 
};

enum scal_curve {
	NEG_LIN = 0,
	NEG_EXP, 
	POS_EXP, 
	POS_LIN
};

enum par_type {
	PAR_AMS = 0,
	PAR_MODE,
	PAR_SYNC,
	PAR_COARSE,
	PAR_FINE,
	PAR_FIXED,
	PAR_AEGRATE,
	PAR_AEGLEVEL,
	PAR_BREAKPT,
	PAR_CL,
	PAR_CR,
	PAR_DL,
	PAR_DR,
	PAR_RS,
	PAR_OLEVEL,
	PAR_KVS,
	PAR_ENABLED,
	PAR_ALG,
	PAR_FB,
	PAR_LFOWAVE,
	PAR_LFOSPEED,
	PAR_LFODELAY,
	PAR_LFOSYNC,
	PAR_PMS,
	PAR_PEGRATE,
	PAR_PEGLEVEL,
	PAR_TRANSPOSE
};

typedef struct oscillator {
	u32 ams; /* Amplitude Modulation Sensitivity */
	u32 mode; // 0 = ratio
	u32 sync;
	u32 freq_coarse;
	u32 freq_fine;
	u32 freq_fixed; // pacc value
	u32 detune;		// 8 = no detuning, 15 = max (+7)
	u32 eg_rate[4];
	u32 eg_lvl[4];
	u32 break_pt;
	u32 curve_left;
	u32 curve_right;
	u32 depth_left;
	u32 depth_right;
	u32 rate_scaling;
	u32 output_lvl;
	u32 kvs; /* Key Velocity Sensitivity */ // 12 bits
	u32 enabled;
} oscillator_t;

typedef struct patch {
	u32 alg;
	u32 fb;
	u32 lfo_wave;
	u32 lfo_speed;
	u32 lfo_delay;
	u32 lfo_sync;
	u32 pms; /* Pitch Modulation Sensitivity */
	u32 pitch_eg_rate[4];
	u32 pitch_eg_lvl[4];
	u32 key_transpose;
	oscillator_t osc[6];
} patch_t;


typedef struct synth {
	// pressed velocities needs to be stored as well as PB range
	// in order to calculate new frequencies and outlevels "on the fly"
	u8 base_ch;
	u8 next_ch;
	u8 no_ch_avail;
	u8 sustain;
	u8 ch_array[16]; //u8 will be much faster to code with, maybe uglier but who cares?
	u16 pitch_bend;
	u16 patch_no;
	patch_t patch;	
} synth_t;

void synth_send_pitch(synth_t* synth_handle, u8 channel);
void synth_refresh_hw(synth_t* synth_handle);
void synth_testpatch(synth_t* synth_handle);
void synth_init(synth_t* synth_handle, u8 polyphony, u8 base_ch);
void synth_noteoff(synth_t* synth_handle, u8 notevalue, u8 velocity);
void synth_noteon(synth_t* synth_handle, u8 notevalue, u8 velocity);
void synth_cc(synth_t* synth_handle, u8 parameter, u8 value);
void synth_pc(synth_t* synth_handle, u8 patch_nbr);
void synth_pw(synth_t* synth_handle, u16 value);
void synth_decodesysex(u16 SysexRcvIndex);
void synth_sendvolume(u8 value);
void synth_sendpan(u8 value);
void synth_setparameter(synth_t* synth_handle, u8 op, u8 stage, u32 parType, u32 value);

#endif 
