#include "usbdec.h"
#include "midi.h"
#include "synth.h"

/* Note Event Functions */
void sendUNoteOn() {
	u8 note =  recvValue[0];
	u8 velocity =  recvValue[1];
	synth_noteon(midi_getusbdevice(), note, velocity);
	/* Debug
	xil_printf("sendUNoteOn(): synth_noteon(%x, %x, %x);\r\n", midi_getusbdevice(), note, velocity);
	*/
}
void sendUNoteOff() {
	u8 note =  recvValue[0];
	u8 velocity =  recvValue[1];
	synth_noteoff(midi_getusbdevice(), note, velocity);
	/* Debug
	xil_printf("sendUNoteOff(): synth_noteoff(%x, %x, %x);\r\n", midi_getusbdevice(), note, velocity); 
	*/
}

/* Global Parameter Functions */
void sendGlVolume() {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_sendvolume(value >> 24); // volume is 8 bits
	/* Debug
	xil_printf("sendGlVolume(): synth_sendvolume(%x);\r\n", value >> 24);
	*/
}

void sendPan() {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_sendpan(value >> 24); // pan is 8 bits
	/* Debug
	xil_printf("sendPan(): synth_sendpan(%x);\r\n", value >> 24);
	*/
}

void sendAlgorithm() {
	u8 value = recvValue[0]; // 0..31
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_ALG, value);
	/* Debug
	xil_printf("sendAlgorithm(): synth_setparameter(%x, 0, 0, PAR_ALG, %x);\r\n", midi_getusbdevice(), value);
	*/
}

void sendFeedback() {
	u8 value = recvValue[0]; // 0..7
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_FB, value);
	/* Debug
	xil_printf("sendFeedback(): synth_setparameter(%x, 0, 0, PAR_FB, %x);\r\n", midi_getusbdevice(), value);
	*/
}

void sendLFOWave() {
	u8 value = recvValue[0]; // 0..5
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_LFOWAVE, value);
	/* Debug
	xil_printf("sendLFOWave(): synth_setparameter(%x, 0, 0, PAR_LFOWAVE, %x);\r\n", midi_getusbdevice(), value);
	*/
}

void sendLFOSpeed() {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_LFOSPEED, value >> 20); //lfo speed is 12 bits
	/* Debug
	xil_printf("sendLFOSpeed(): synth_setparameter(%x, 0, 0, PAR_LFOSPEED, %x);\r\n", midi_getusbdevice(), value >> 20);
	*/
}

void sendLFODelay() {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_LFODELAY, value >> 12); //lfo delay is 20 bits
	/* Debug
	xil_printf("sendLFODelay(): synth_setparameter(%x, 0, 0, PAR_LFODELAY, %x);\r\n", midi_getusbdevice(), value >> 12);
	*/
}

void sendLFOSync() {
	u8 value = recvValue[0]; // 0, 1 (Off, On) 
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_LFOSYNC, value);
	/* Debug
	xil_printf("sendLFOSync(): synth_setparameter(%x, 0, 0, PAR_LFOSYNC, %x);\r\n", midi_getusbdevice(), value);
	*/
}

void sendLFOPitch() {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_PMS, value >> 20); //pms is 12 bits
	/* Debug
	xil_printf("sendLFOPitch(): synth_setparameter(%x, 0, 0, PAR_PMS, %x);\r\n", midi_getusbdevice(), value >> 20);
	*/
}

void sendOSCSync() {
	u8 i;
	u8 value = recvValue[0]; // 0, 1 (Off, On)
	for(i = 0; i < 6; ++i)
		synth_setparameter(midi_getusbdevice(), i, 0, PAR_SYNC, value);
	/* Debug
	xil_printf("sendOSCSync(): synth_setparameter(%x, op, 0, PAR_SYNC, %x);\r\n", midi_getusbdevice(), value);
	*/
}

void sendPEGRate(u8 egStage) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), 0, egStage, PAR_PEGRATE, value);
	/* Debug
	xil_printf("sendPEGLevel(): synth_setparameter(%x, 0, %x, PAR_PEGRATE, %x);\r\n", midi_getusbdevice(), egStage, value);
	*/
}

void sendPEGLevel(u8 egStage) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), 0, egStage, PAR_PEGLEVEL, value >> 20); //eg level is 12 bits
	/* Debug
	xil_printf("sendPEGRate(): synth_setparameter(%x, 0, %x, PAR_PEGLEVEL, %x);\r\n", midi_getusbdevice(), egStage, value >> 20);
	*/
}

void sendTranspose() {
	u8 value = recvValue[0]; // 0..24..48 (-24..0..+24) 
	synth_setparameter(midi_getusbdevice(), 0, 0, PAR_TRANSPOSE, value);
	/* Debug
	xil_printf("sendTranspose(): synth_setparameter(%x, 0, 0, PAR_TRANSPOSE, %x);\r\n", midi_getusbdevice(), value);
	*/
}

/* Operator Parameter Functions */
void sendOpVolume(u8 op) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(),op,0,PAR_OLEVEL, value >> 20); //outlevel is 12 bits
	/* Debug
	xil_printf("sendOpVolume(): synth_setparameter(%x, %x, 0, PAR_OLEVEL, %x);\r\n", midi_getusbdevice(), op, value >> 20);
	*/
}

void sendActive(u8 op) {
	u8 value = recvValue[0]; // 0, 1 (Off, On) 
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_ENABLED, value);
	/* Debug
	xil_printf("sendActive(): synth_setparameter(%x, %x, 0, PAR_ENABLED, %x);\r\n", midi_getusbdevice(), op, value);
	*/
}

void sendLFOAMS(u8 op) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_AMS, value >> 20); // LFO Ams is 12 bits
	/* Debug
	xil_printf("sendLFOAMS(): synth_setparameter(%x, %x, 0, PAR_AMS, %x);\r\n", midi_getusbdevice(), op, value >> 20);
	*/
	
}

void sendOSCMode(u8 op) {
	u8 value = recvValue[0]; // 0, 1 (Ratio, Fixed) 
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_MODE, value);
	/* Debug
	xil_printf("sendOSCMode(): synth_setparameter(%x, %x, 0, PAR_MODE, %x);\r\n", midi_getusbdevice(), op, value);
	*/
}

void sendOSCRatioCoarse(u8 op) {
	u8 value = recvValue[0]; // 1, 2, 4..64
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_COARSE, value);
	/* Debug
	xil_printf("sendOSCRatioCoarse(): synth_setparameter(%x, %x, 0, PAR_COARSE, %x);\r\n", midi_getusbdevice(), op, value);
	*/
}

void sendOSCRatioFine(u8 op) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_FINE, ((value >> 7) * 100) >> 25);
	/* Debug
	xil_printf("sendOSCRatioFine(): synth_setparameter(%x, %x, 0, PAR_FINE, %x);\r\n", midi_getusbdevice(), op, ((value >> 7) * 100) >> 24);
	*/
}

void sendOSCFixedFreq(u8 op) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16; // 100 - 1000000 (1 Hz - 1 kHz in steps of 0.01 Hz)
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_FIXED, value);
}

void sendEGRate(u8 op, u8 egStage) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), op, egStage, PAR_AEGRATE, value >> 8); // any value higher would be useless anyway
	/* Debug
	xil_printf("sendEGRate(): synth_setparameter(%x, %x, %x, PAR_AEGRATE, %x);\r\n", midi_getusbdevice(), op, egStage, value);
	*/
}

void sendEGLevel(u8 op, u8 egStage) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), op, egStage, PAR_AEGLEVEL, value >> 20);  //eg level is 12 bits
	/* Debug
	xil_printf("sendEGLevel(): synth_setparameter(%x, %x, %x, PAR_AEGLEVEL, %x);\r\n", midi_getusbdevice(), op, egStage, value >> 20);
	*/
}

void sendRateScale(u8 op) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_AEGLEVEL, value);
}

void sendVelSense(u8 op) {
	u32 value =   ((u32) recvValue[0]) 
					| ((u32) recvValue[1]) << 8 
					| ((u32) recvValue[2]) << 16 
					| ((u32) recvValue[3]) << 24;
	synth_setparameter(midi_getusbdevice(), op, 0, PAR_AEGLEVEL, value);
}

/* Decoding Functions */
void sendNoteEvent() {
	switch(typeID) {
		case N_OFF:
			sendUNoteOff();
			break;
		case N_ON:
			sendUNoteOn();
			break;
	}
}

void sendGlobalEvent() {
	switch(typeID) {
		case GLB_VOL:
			sendGlVolume();
			break;
		case GLB_PAN:
			sendPan();
			break;
		case GLB_ALG:
			sendAlgorithm();
			break;
		case GLB_FBK:
			sendFeedback();
			break;
		case GLB_LFO_WAV:
			sendLFOWave();
			break;
		case GLB_LFO_SPD:
			sendLFOSpeed();
			break;
		case GLB_LFO_DEL:
			sendLFODelay();
			break;
		case GLB_LFO_SYNC:
			sendLFOSync();
			break;
		case GLB_LFO_PITCH:
			sendLFOPitch();
			break;
		case GLB_OSC_SYNC:
			sendOSCSync();
			break;
		case GLB_PEG_R1:
			sendPEGRate(1);
			break;
		case GLB_PEG_L1:
			sendPEGLevel(1);
			break;
		case GLB_PEG_R2:
			sendPEGRate(2);
			break;
		case GLB_PEG_L2:
			sendPEGLevel(2);
			break;
		case GLB_PEG_R3:
			sendPEGRate(3);
			break;
		case GLB_PEG_L3:
			sendPEGLevel(3);
			break;
		case GLB_PEG_R4:
			sendPEGRate(0);
			break;
		case GLB_PEG_L4:
			sendPEGLevel(0);
			break;
		case GLB_TRANS:
			sendTranspose();
			break;
	}
}

void sendOperatorEvent(u8 op) {
	switch(typeID) {
		case OP_VOL:
			sendOpVolume(op);
			break;
		case OP_ACT:
			sendActive(op);
			break;
		case OP_LFO_AMS:
			sendLFOAMS(op);
			break;
		case OP_OSC_MODE:
			sendOSCMode(op);
			break;
		case OP_OSC_RATIO_COARSE:
			sendOSCRatioCoarse(op);
			break;
		case OP_OSC_RATIO_FINE:
			sendOSCRatioFine(op);
			break;
		case OP_OSC_FIXED_FREQ:
			sendOSCFixedFreq(op);
			break;
		case OP_EG_R1:
			sendEGRate(op, 0);
			break;
		case OP_EG_L1:
			sendEGLevel(op, 0);
			break;
		case OP_EG_R2:
			sendEGRate(op, 1);
			break;
		case OP_EG_L2:
			sendEGLevel(op, 1);
			break;
		case OP_EG_R3:
			sendEGRate(op, 2);
			break;
		case OP_EG_L3:
			sendEGLevel(op, 2);
			break;
		case OP_EG_R4:
			sendEGRate(op, 3);
			break;
		case OP_EG_L4:
			sendEGLevel(op, 3);
			break;
		case OP_KBD_RS:
			sendRateScale(op);
			break;
		case OP_KDB_VS:
			sendVelSense(op);
			break;
	}
}

void decodeUsb() {

	eventID = RecvUsbBuffer[usbInReadItr].data[0];
	typeID = RecvUsbBuffer[usbInReadItr].data[1];
	recvValue[0] = RecvUsbBuffer[usbInReadItr].data[2];
	recvValue[1] = RecvUsbBuffer[usbInReadItr].data[3];
	recvValue[2] = RecvUsbBuffer[usbInReadItr].data[4];
	recvValue[3] = RecvUsbBuffer[usbInReadItr].data[5];
	usbInReadItr = incIt(usbInReadItr);
	
	switch(eventID) {
		case NOTE_EVENT:
			sendNoteEvent();
			break;
		case GLOBAL_PARAM:
			sendGlobalEvent();
			break;
		case OP1_PARAM:
		case OP2_PARAM:
		case OP3_PARAM:
		case OP4_PARAM:
		case OP5_PARAM:
		case OP6_PARAM:
			sendOperatorEvent(6-eventID); // operator 6 is encoded as 0, and so on...
			break;
		default:
			/* If Error Event Received */
			break;
	}
}
