----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:27:59 10/03/2011 
-- Design Name: 
-- Module Name:    FSL_CONTROLLER - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.ops_package.all;
use work.dx7_hw_top_package.all;
use work.types.all;

entity dx7_hw_top is
	port(		
		clk      : in std_logic; -- clock
		rst_n    : in std_logic; -- reset, active low
		-- read signals
		exists   : in std_logic; -- active if data is available
		rd_ack   : out std_logic; -- read ack from the core
		d_in     : in std_logic_vector(0 to 31); -- data to the core
		-- write signals
		full     : in std_logic; -- active if fifo is full
		wr_en    : out std_logic; -- read ack from the core
		d_out    : out std_logic_vector(0 to 31); -- data to the core
		-- i2s signals
		i2s_mclk     : out std_logic;
		i2s_lrck     : out std_logic;
		i2s_sclk     : out std_logic;
		i2s_dout     : out std_logic;
		DAC_MCLK     : in std_logic
	);
end dx7_hw_top;

architecture behavioral of dx7_hw_top is
	signal rst : std_logic;
	signal egs_wr : std_logic;
	signal ops_wr : std_logic;
	signal vol_wr : std_logic;
	signal pan_wr : std_logic;
	signal egs_hold : std_logic;
	
	signal ops_sync : std_logic;
	signal ops_key_sync : std_logic;
	signal ops_freq : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal ops_amp : std_logic_vector(AMP_WIDTH-1 downto 0);
	
	signal mixer_data : std_logic_vector(SAMPLE_WIDTH-1 downto 0);
	signal mixer_vol : std_logic_vector(CTRL_WIDTH-1 downto 0);
	signal mixer_pan : std_logic_vector(CTRL_WIDTH-1 downto 0);
	signal i2s_l_out : std_logic_vector(DAC_WIDTH-1 downto 0);
	signal i2s_r_out : std_logic_vector(DAC_WIDTH-1 downto 0);
	
	signal i2s_ctrl_mclk : std_logic;
	signal i2s_ctrl_lr_clk : std_logic;
	signal i2s_ctrl_data : std_logic;
	signal i2s_ctrl_sclk : std_logic;
	
	signal data24 : std_logic_vector(23 downto 0);
	signal fullvol : std_logic_vector(7 downto 0);
	signal cpan : std_logic_vector(7 downto 0);
	
	begin
		rst <= not rst_n; -- All components use active high reset
		
		fsl_controller_inst : fsl_controller port map(clk, rst, exists, rd_ack, d_in, full, wr_en, d_out, egs_wr, ops_wr, vol_wr, pan_wr, egs_hold);
		
		egs_inst : egs port map(clk, rst, i2s_ctrl_lr_clk, ops_sync, ops_key_sync, ops_freq, ops_amp, d_in , egs_wr, egs_hold);
		
		ops_inst : ops_hw port map(clk, rst, ops_sync, ops_key_sync, ops_freq, ops_amp, d_in(24 to 31), d_in(0 to 3), ops_wr, mixer_data);
		
		vol_reg : en_dff generic map(CTRL_WIDTH) port map(clk, rst, vol_wr, d_in(31-CTRL_WIDTH+1 to 31), mixer_vol);
		
		pan_reg : en_dff generic map(CTRL_WIDTH) port map(clk, rst, pan_wr, d_in(31-CTRL_WIDTH+1 to 31), mixer_pan);
		
		mixer_inst : mixer generic map(SAMPLE_WIDTH, DAC_WIDTH, CTRL_WIDTH) 
								 port map(clk, rst, mixer_data, i2s_l_out, i2s_r_out, mixer_vol, mixer_pan); 
														
		i2sctrl_inst : i2sctrl port map(DAC_MCLK, rst, i2s_l_out, i2s_r_out, i2s_ctrl_mclk, i2s_ctrl_lr_clk, i2s_ctrl_data, i2s_ctrl_sclk);
		
		i2s_mclk <= i2s_ctrl_mclk;
		i2s_lrck <= i2s_ctrl_lr_clk;
		i2s_sclk <= i2s_ctrl_sclk;
		--i2s_sclk <= '0'; -- for internal clock mode
		i2s_dout <= i2s_ctrl_data;
end behavioral;