--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use work.ops_package.all;
use work.egs_package.all;

package dx7_hw_top_package is

-- FSL Controller
component fsl_controller
	port(
		clk      : in std_logic; -- clock
		rst      : in std_logic; -- reset
		-- read signals
		exists   : in std_logic; -- active if data is available
		rd_ack   : out std_logic; -- read ack from this core
		d_in     : in std_logic_vector(31 downto 0); -- data to this core
	   -- write signals
		full     : in std_logic; -- active if fifo is full
		wr_en    : out std_logic; -- read ack from this core
		d_out    : out std_logic_vector(31 downto 0); -- data from this core
		-- internal signals
		egs_wr   : out std_logic;
		ops_wr   : out std_logic;
		vol_wr   : out std_logic;
		pan_wr   : out std_logic;
		egs_hold : in std_logic
	);
end component;

-- EGS
component egs
   Port ( clk : in  STD_LOGIC;
          rst : in  STD_LOGIC;
          lrck_in : in  STD_LOGIC;
          sync_out : out  STD_LOGIC;
          osc_sync : out  STD_LOGIC;
          freq : out  STD_LOGIC_VECTOR (PHASEWIDTH-1 downto 0);
          amp : out  STD_LOGIC_VECTOR (AMPWIDTH-1 downto 0);
          fsl_data : in  STD_LOGIC_VECTOR (31 downto 0);
          fsl_wen : in  STD_LOGIC;
			 hold : out  STD_LOGIC);
end component;
  
-- OPS
component ops_hw
    port ( clk : in std_logic;
			  rst : in std_logic;
           sync : in std_logic;
			  key_sync : in std_logic;
           phase_inc : in std_logic_vector (FREQ_WIDTH-1 downto 0);
           amp_mod : in std_logic_vector (AMP_WIDTH-1 downto 0);
           alg_data : in std_logic_vector (7 downto 0);
			  alg_ch : in std_logic_vector(3 downto 0);
           alg_wr : in std_logic;
           sample_data : out std_logic_vector (SAMPLE_WIDTH-1 downto 0));
end component;

-- Mixer
component mixer
	generic(
		IN_BITS : integer := 16;
		OUT_BITS : integer := 16;
		CTRL_BITS : integer range 1 to 16 := 8
	);
    Port ( clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           a_in : in  STD_LOGIC_VECTOR (IN_BITS-1 downto 0);
           l_out : out  STD_LOGIC_VECTOR (OUT_BITS-1 downto 0);
			  r_out : out  STD_LOGIC_VECTOR (OUT_BITS-1 downto 0);
           vol : in  STD_LOGIC_VECTOR (CTRL_BITS-1 downto 0);
           pan : in  STD_LOGIC_VECTOR (CTRL_BITS-1 downto 0));
end component;

-- I2S Controller
component i2sctrl is
	generic (
		MBITS : integer := 10; -- Width for mclk counter
		SBITS : integer := 4; -- width for sclk counter
		DBITS : integer := 24; -- Audio Data bits
		MLRCK : integer := 768; -- mclk / lrck ratio
		MSCLK : integer := 16 -- mclk / sclk ratio
	);
    Port ( clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           data_l : in  STD_LOGIC_VECTOR (DBITS-1 downto 0);
           data_r : in  STD_LOGIC_VECTOR (DBITS-1 downto 0);
           mclk_out : out  STD_LOGIC;
           lrck_out : out  STD_LOGIC;
           dac_out : out  STD_LOGIC;
           sclk_out : out  STD_LOGIC);
end component;
 
end dx7_hw_top_package;
