----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:27:59 10/03/2011 
-- Design Name: 
-- Module Name:    FSL_CONTROLLER - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity fsl_controller is
	port(
		clk      : in std_logic; -- clock
		rst      : in std_logic; -- reset
		-- read signals
		exists   : in std_logic; -- active if data is available
		rd_ack   : out std_logic; -- read ack from this core
		d_in     : in std_logic_vector(31 downto 0); -- data to this core
	   -- write signals
		full     : in std_logic; -- active if fifo is full
		wr_en    : out std_logic; -- read ack from this core
		d_out    : out std_logic_vector(31 downto 0); -- data from this core
		-- internal signals
		egs_wr   : out std_logic;
		ops_wr   : out std_logic;
		vol_wr   : out std_logic;
		pan_wr   : out std_logic;
		egs_hold : in std_logic
	);
end fsl_controller;

architecture behavioral of fsl_controller is
	type fsl_state_t is (s0, s1);
	type fsl_rcd is record
		fsl_state : fsl_state_t;
		read_ack : std_logic;
		egs : std_logic;
		ops : std_logic;
		vol : std_logic;
		pan : std_logic;
	end record;
	signal r, r_next : fsl_rcd;
	
	begin
	wr_en <= '0';
	d_out <= (others=>'0');
	
	process(clk)
		begin
			if(clk'event and clk='1') then
				r <= r_next;
			end if;
	end process;


	-- we have a mealy fsl controller. That can cause trouble, right?
	process(rst, exists, d_in, egs_hold, r)
		variable v : fsl_rcd;
		begin
			v := r;
			v.read_ack := '0';
			v.egs := '0';
			v.ops := '0';
			v.vol := '0';
			v.pan := '0';
			case (r.fsl_state) is 
				when s0 => if(exists = '1') then
									if(d_in(27 downto 25) = "111") then -- Data for OPS/Mixer
										case (d_in(24 downto 23)) is
											when "01" => v.ops := '1';																 
											when "10" => v.vol := '1';	
											when "11" => v.pan := '1';
											when others => -- Unknown data!!!
										end case;								
										v.read_ack := '1';
										v.fsl_state := s1;
									else -- Data for EGS
										v.egs := '1';
										if(egs_hold = '1') then
											v.read_ack := '0';
											v.fsl_state := s0;
										else
											v.read_ack := '1';
											v.fsl_state := s1;
										end if;
									end if;									
							  end if;
				
				when s1 => v.fsl_state := s0;
			end case;
			
			if(rst = '1') then
				v.fsl_state := s0;
				v.read_ack := '0';
				v.egs := '0';
				v.ops := '0';
				v.vol := '0';
				v.pan := '0';
			end if;
			r_next <= v;
	end process;
	egs_wr <= r_next.egs;
	ops_wr <= r_next.ops;
	vol_wr <= r_next.vol;
	pan_wr <= r_next.pan;
	rd_ack <= r_next.read_ack;

end behavioral;

