----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:05:46 09/15/2011 
-- Design Name: 
-- Module Name:    OPS_Components - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity ops_dff is
	generic(
		width : integer
	);
	port(
		clk, rst : in std_logic;
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end ops_dff;

architecture behavioral of ops_dff is
	begin
		process(clk)
			begin
				if(clk'event and clk = '1') then
					if(rst = '1') then
						q <= (others => '0');
					else
						q <= d;
					end if;
				end if;
		end process;
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity b_dff is
	port(
		clk, rst : in std_logic;
		d : in std_logic;
		q : out std_logic
	);
end b_dff;

architecture behavioral of b_dff is
	begin
		process(clk)
			begin
				if(clk'event and clk = '1') then
					if(rst = '1') then
						q <= '0';
					else
						q <= d;
					end if;
				end if;
		end process;
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;

entity en_dff is
  generic(
    width : integer
  );
  port(
    clk, rst, en : in std_logic;
    d : in std_logic_vector(width-1 downto 0);
    q : out std_logic_vector(width-1 downto 0)
  );
end en_dff;

architecture behavioral of en_dff is
  begin
    process(clk)
      begin
		  if(clk'event and clk = '1') then
			if(rst = '1') then
				q <= (others => '0');
         elsif(en = '1') then
            q <= d;
         end if;
        end if;
    end process;    
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;

entity n_en_dff is
  generic(
    width : integer
  );
  port(
    clk, rst, en : in std_logic;
    d : in std_logic_vector(width-1 downto 0);
    q : out std_logic_vector(width-1 downto 0)
  );
end n_en_dff;

architecture behavioral of n_en_dff is
  begin
    process(clk)
      begin
		  if(clk'event and clk = '0') then
			if(rst = '1') then
				q <= (others => '0');
         elsif(en = '1') then
            q <= d;
         end if;
        end if;
    end process;    
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity xdelay is
	generic(
		width : integer;
		depth : integer
	);
	port(
		clk : in std_logic;
		rst : in std_logic;
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end xdelay;

architecture behavioral of xdelay is
	type d_array_t is array (depth-1 downto 0) of std_logic_vector(width-1 downto 0);
	signal d_array : d_array_t;
	begin
		process(clk)
			begin
				if(clk'event and clk='1') then
					if(rst='1') then
						for i in depth-1 downto 0 loop
							d_array(i) <= (others => '0');
						end loop;
					else
						d_array(depth-1) <= d;
						for i in depth-2 downto 0 loop
							d_array(i) <= d_array(i+1);
						end loop;
					end if;
				end if;
		end process;
		q <= d_array(0);
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity reg_file is
	generic(
		width : integer
	);
	port(
		clk, rst, wr: in std_logic;
		addr : in std_logic_vector(3 downto 0); -- 16 channels
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end reg_file;

architecture behavioral of reg_file is

	type regf_t is array (15 downto 0) of std_logic_vector(width-1 downto 0);
	signal regf : regf_t;	
	signal r_adr, w_adr : integer;
	
	begin
	
		r_adr <= conv_integer(unsigned(addr));
		w_adr <= conv_integer(unsigned(addr-1));
	
		write: process(clk)    
		begin
			if(clk'event and clk = '1') then
				if(rst = '1') then
					for i in 15 downto 0 loop
						regf(i) <= (others => '0');
					end loop;
				elsif(wr = '1') then
					regf(w_adr) <= d;
				end if;
			end if;
		end process;		
		
		read: process(clk) 
		begin
			if(clk'event and clk = '1') then
				q <= regf(r_adr);
			end if;
		end process;
		
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity ch_rst_reg_file is
	generic(
		width : integer
	);
	port(
		clk, rst, wr, ch_rst: in std_logic;
		addr : in std_logic_vector(3 downto 0); -- 16 channels
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end ch_rst_reg_file;

architecture behavioral of ch_rst_reg_file is

	type regf_t is array (15 downto 0) of std_logic_vector(width-1 downto 0);
	signal regf : regf_t;	
	signal r_adr, w_adr : integer;
	
	begin
	
		r_adr <= conv_integer(unsigned(addr));
		w_adr <= conv_integer(unsigned(addr-1));
	
		write: process(clk)
		begin
			if(clk'event and clk = '1') then
				if(rst = '1') then					
					for i in 15 downto 0 loop
						regf(i) <= (others => '0');
					end loop;
				elsif(ch_rst = '1') then	
					regf(r_adr) <= (others => '0');
				elsif(wr = '1') then
					regf(w_adr) <= d;
				end if;
			end if;
		end process;		
		
		read: process(clk)
		begin
			if(clk'event and clk = '1') then
				q <= regf(r_adr);
			end if;
		end process;
		
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use work.ops_package.all;

entity shifter is
	port(
		in_data : in std_logic_vector(PHASE_WIDTH-1 downto 0);
		fb_lvl : in std_logic_vector(2 downto 0);
		out_data : out std_logic_vector(PHASE_WIDTH-1 downto 0)
	);
end shifter;

architecture behavioral of shifter is
	signal zeros : std_logic_vector(PHASE_WIDTH-9 downto 0);
	signal fb : std_logic_vector(2 downto 0);
	signal shifted_data : std_logic_vector(7 downto 0);
	signal seven : std_logic_vector(2 downto 0);
	begin
		-- feedback level is applied only to 8 MSB bits of in data & FREQ_WIDTH-8 LSB bits are taken 0
		zeros <= (others => '0');
		seven <= "111";
		fb <= unsigned(seven) - unsigned(fb_lvl);
		shifted_data <= conv_std_logic_vector(shr(signed(in_data(PHASE_WIDTH-1 downto PHASE_WIDTH-8)), unsigned(fb)), 8);
		out_data <= shifted_data & zeros;
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.ops_package.all;

entity phase_offset_normalizer is
	port(
		clk : in std_logic;
		input : in std_logic_vector(PHASE_WIDTH-1 downto 0);
		output : out std_logic_vector(FREQ_WIDTH-1 downto 0)
	);
end phase_offset_normalizer;

architecture behavioral of phase_offset_normalizer is
--	signal temp : integer;
--	signal normalized_input : std_logic_vector(PHASE_WIDTH+NORM_WIDTH-1 downto 0);
	signal phase_offset : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal phase_zeros : std_logic_vector(FREQ_WIDTH-PHASE_WIDTH-1+3 downto 0);
	
	begin
		
--		temp <= conv_integer(signed(input))*norm_value;		
--		normalized_input <= conv_std_logic_vector(temp, phase_width+norm_width);	
--		phase_offset <= normalized_input(phase_width+norm_width-4 downto phase_width+norm_width-freq_width-3);


-- 	Modulation precision is lost with reduced number of bits & appended zeros
		phase_zeros <= (others=>'0');
		phase_offset <= input(PHASE_WIDTH-4 downto 0) & phase_zeros;
		
		sync : process(clk)
		begin
			if(clk'event and clk = '1') then
				output <= phase_offset;
			end if;
		end process;
		
--		phase_mantissa <= normalized_input(FREQ_WIDTH+NORM_WIDTH-1) & normalized_input(FREQ_WIDTH+NORM_WIDTH-4 downto FREQ_WIDTH-NORM_WIDTH+2);
--		
--		with(phase_mantissa(FREQ_WIDTH-1) xor phase_mantissa(FREQ_WIDTH-2)) select
--			output_next <= ((not phase_mantissa(FREQ_WIDTH-1)) & phase_mantissa(FREQ_WIDTH-2 downto 0)) when '1',
--			phase_mantissa when others;		

end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;

entity alg_reg_file is
	port(
		clk, rst, wr: in std_logic;
		wr_addr : in std_logic_vector(3 downto 0); -- 16 channels
		rd_addr : in std_logic_vector(3 downto 0); -- 16 channels
		d : in std_logic_vector(7 downto 0);
		q : out std_logic_vector(7 downto 0)
	);
end alg_reg_file;

architecture behavioral of alg_reg_file is

	type regf_t is array (15 downto 0) of std_logic_vector(7 downto 0);
	signal regf : regf_t;	
	signal wr_adr, rd_adr : integer;
	
	begin
	
		wr_adr <= conv_integer(unsigned(wr_addr));
		rd_adr <= conv_integer(unsigned(rd_addr));
	
		write: process(clk)         
		begin
			if(clk'event and clk = '1') then
				if(rst = '1') then
					for i in 15 downto 0 loop
						regf(i) <= (others => '0');
					end loop;
				elsif(wr = '1') then
					regf(wr_adr) <= d;
				end if;
			end if;
		end process;
		
		read: process(clk)
		begin
			if(clk'event and clk = '1') then
				q <= regf(rd_adr);
			end if;
		end process;		
		
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.ops_package.all;

entity op_ctrl is
	port(
		clk : in std_logic;
		rst : in std_logic;
		sync : in std_logic;
		reg_ch : out std_logic_vector(3 downto 0);
		alg_reg_ch : out std_logic_vector(3 downto 0);
		op_en : out std_logic;
		alg_col : out std_logic_vector(2 downto 0);
		comp_col : out std_logic_vector(2 downto 0)
	);
end op_ctrl;

architecture behavioral of op_ctrl is
	type state_t is (s0, s1);
	type ctrl_record is record
		state : state_t;
		op_en : std_logic;
		alg_ctr : std_logic_vector(6 downto 0);
		comp_ctr : std_logic_vector(6 downto 0);
	end record;
	signal r, r_next : ctrl_record;
	signal op_en_delay : std_logic;
	
	begin
		synchronous : process(clk)
			begin
				if(clk'event and clk = '1') then -- OP ctrl operates on positive edge
					r <= r_next;
				end if;
		end process;	
	
		combinational : process(r, rst, sync)
			variable v : ctrl_record;
			begin
				v := r;
				case(r.state) is
					when s0 => if(sync = '1') then 
										v.state := s1;
										v.op_en := '1';
										v.alg_ctr := "0010000";
										v.comp_ctr := "0010010";
									end if;
					when s1 => if(r.alg_ctr >= 127) then
										v.state := s0;
										v.alg_ctr := (others=>'0');
										v.comp_ctr := (others=>'0');
								  else
										v.alg_ctr := v.alg_ctr + 1;
										v.comp_ctr := v.comp_ctr + 1;
								  end if;
								  if(r.alg_ctr >= 111) then
										v.op_en := '0';
								  else
										v.op_en := '1';
								  end if;
				end case;			
				if(rst = '1') then
					v.state := s0;
					v.op_en := '0';
					v.alg_ctr := (others=>'0');
					v.comp_ctr := (others=>'0');
				end if;				
				r_next <= v;
		end process;
		alg_col <= r.alg_ctr(6 downto 4);
		reg_ch <= r.alg_ctr(3 downto 0);
		alg_reg_ch <= r_next.alg_ctr(3 downto 0);
		comp_col <= r.comp_ctr(6 downto 4);
		D1 : b_dff port map(clk, rst, r.op_en, op_en_delay);
		D2 : b_dff port map(clk, rst, op_en_delay, op_en);
end behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use work.ops_package.all;

entity alg_rom is
	port(
		clk : in std_logic;
		alg_row : in std_logic_vector(4 downto 0);
		alg_col : in std_logic_vector(2 downto 0);
		comp_row : in std_logic_vector(4 downto 0);
		comp_col : in std_logic_vector(2 downto 0);
		oren : out std_logic;
		mren : out std_logic;
		fren : out std_logic;
		mod_sel : out std_logic_vector(2 downto 0);
		comp : out std_logic_vector(7 downto 0)
	);
end alg_rom;

architecture behavioral of alg_rom is
	signal a_r, a_c, c_r, c_c : integer;
	begin
		a_r <= conv_integer(unsigned(alg_row));
		a_c <= conv_integer(unsigned(alg_col));
		c_r <= conv_integer(unsigned(comp_row));
		c_c <= conv_integer(unsigned(comp_col));
		
		process(clk)
			begin
				if(clk'event and clk = '1') then
					oren <= oren_table(a_r, a_c);
					mren <= mren_table(a_r, a_c);
					fren <= fren_table(a_r, a_c);
					mod_sel <= mod_sel_table(a_r, a_c);
					comp <= comp_table(c_r, c_c);
				end if;
		end process;
end behavioral;


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.ops_package.all;

entity phase_accu is
	port(
		clk : in std_logic;
		rst : in std_logic;
		ch_sclr : in std_logic;
		ch_wr : in std_logic;
		pinc : in std_logic_vector(FREQ_WIDTH-1 downto 0);
		poff : in std_logic_vector(FREQ_WIDTH-1 downto 0);
		pout : out std_logic_vector(FREQ_WIDTH-1 downto 0)
	);
end phase_accu;

architecture behavioral of phase_accu is

	signal ch_curr : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal ch_curr_sync : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal ch_next : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal ch_inc : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal ch_out : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal poff_delay : std_logic_vector(FREQ_WIDTH-1 downto 0);
	signal ch_wr_delay : std_logic;
	signal ch : std_logic_vector(6 downto 0);
	signal mem_spo : std_logic_vector(FREQ_WIDTH-1 downto 0);
	
	type state_t is (s0, s1);
	type ctr_record is record
		state : state_t;
		ctr : std_logic_vector(7 downto 0); -- 8 bits because using signed operations!
	end record;
	signal r, r_next : ctr_record;	

	begin
		
		with ch_sclr select 
			ch_curr_sync <= (others => '0') when '1',
							    ch_curr when others;	
		ch_next <= pinc + ch_curr_sync;					 
		--accu_inst : pa_mem_dist port map( ch, ch_next, ch, clk, ch_wr, ch_inc, ch_curr);		
		accu_inst : pa_mem_dist port map( ch, ch_next, ch, clk, ch_wr, mem_spo, ch_curr);		
		mem_dff : ops_dff generic map(FREQ_WIDTH) port map(clk, rst, ch_next, ch_inc);
		ch_out <= ch_inc + poff_delay;		
		delay_poff_inst : ops_dff generic map(FREQ_WIDTH) port map(clk, rst, poff, poff_delay);
		delay_ch_wr_inst : b_dff port map(clk, rst, ch_wr, ch_wr_delay);
		delay_pout_inst: en_dff generic map(FREQ_WIDTH) port map(clk, rst, ch_wr_delay, ch_out, pout);

		synchronous : process(clk)
			begin
				if(clk'event and clk = '1') then 
					r <= r_next;
				end if;
		end process;	
	
		combinational : process(r, rst, ch_wr)
			variable v : ctr_record;
			begin
				v := r;
				case(r.state) is
					when s0 => if(ch_wr = '1') then 
										v.state := s1;
										v.ctr := "00000001";
									end if;
					when s1 => if(r.ctr >= "01011111") then -- "01011111" == +95
										v.state := s0;
										v.ctr := (others=>'0');
								  else
										v.ctr := v.ctr + "00000001";
								  end if;
				end case;			
				if(rst = '1') then
					v.state := s0;
					v.ctr := (others=>'0');
				end if;				
				r_next <= v;
		end process;		
		
		ch <= r.ctr(6 downto 0);
end behavioral;