----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:29:02 09/11/2011 
-- Design Name: 
-- Module Name:    OPS_HW - Behavioral 
-- Project Name:   DX7-FPGA
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.ops_package.all;

entity ops_hw is
    port ( clk : in std_logic;
			  rst : in std_logic;
           sync : in std_logic;
			  key_sync : in std_logic;
           phase_inc : in std_logic_vector (FREQ_WIDTH-1 downto 0);
           amp_mod : in std_logic_vector (AMP_WIDTH-1 downto 0);
           alg_data : in std_logic_vector (7 downto 0);
			  alg_ch : in std_logic_vector(3 downto 0);
           alg_wr : in std_logic;
           sample_data : out std_logic_vector (SAMPLE_WIDTH-1 downto 0));
end ops_hw;

architecture Behavioral of OPS_HW is

-- Internal signals / Wires

signal op_en : std_logic;
signal poff_in : std_logic_vector(FREQ_WIDTH-1 downto 0);
signal pout : std_logic_vector(FREQ_WIDTH-1 downto 0);

signal phase_in : std_logic_vector(PHASE_WIDTH-1 downto 0);
signal cosine_data : std_logic_vector(PHASE_WIDTH-1 downto 0);

signal amp : std_logic_vector (AMP_WIDTH-1 downto 0);
signal comp : std_logic_vector(7 downto 0);
signal amp_mod_comp_delay : std_logic_vector(AMP_WIDTH+8-1 downto 0);
signal amp_mod_comp : std_logic_vector(AMP_WIDTH-1 downto 0);
signal mod_data_comp_delay : std_logic_vector(PHASE_WIDTH+AMP_WIDTH-1 downto 0);
signal mod_data_comp : std_logic_vector(PHASE_WIDTH-1 downto 0);
signal m_data_acc : std_logic_vector(PHASE_WIDTH-1 downto 0);
signal m_data : std_logic_vector(PHASE_WIDTH-1 downto 0);
signal m_rst : std_logic;
signal mren : std_logic;

signal reg_ch : std_logic_vector(3 downto 0); -- 16 channels
signal f_reg_addr : std_logic_vector(3 downto 0); -- 16 channels
signal f_data : std_logic_vector(PHASE_WIDTH-1 downto 0);
signal f_data_fl : std_logic_vector(PHASE_WIDTH-1 downto 0);
signal fren : std_logic;
signal fb : std_logic_vector(2 downto 0);
signal fb_delay : std_logic_vector(2 downto 0);

signal mod_sel : std_logic_vector(2 downto 0);
signal poff_in_Norm  : std_logic_vector(PHASE_WIDTH-1 downto 0);

signal o_data_acc : std_logic_vector(SAMPLE_WIDTH-1 downto 0);
signal o_data : std_logic_vector(SAMPLE_WIDTH-1 downto 0);
signal oren : std_logic;

signal alg_reg_ch : std_logic_vector(3 downto 0); -- 16 channels
signal alg_col : std_logic_vector(2 downto 0);
signal comp_col : std_logic_vector(2 downto 0);

signal alg_row : std_logic_vector(4 downto 0);
signal comp_row: std_logic_vector(4 downto 0);

signal alg_sig : std_logic_vector(7 downto 0);

begin

p_accu_inst : phase_accu port map(clk, rst, key_sync, op_en, phase_inc, poff_in, pout);

phase_in <= pout(FREQ_WIDTH-1 downto FREQ_WIDTH-PHASE_WIDTH);

cosine_lut_inst : cosine_lut port map(clk, phase_in, cosine_data);

dealy13_inst : xdelay generic map(width=>AMP_WIDTH, depth=>13) port map(clk, rst, amp_mod, amp); -- amp_mod values are delayed by 13cc

-- comp=128/(number of carriers in an algorithm)
-- 1 => 80
-- 2 => 40
-- 3 => 2B
-- 4 => 20
-- 5 => 1A
-- 6 => 15
amp_mod_comp_delay <= (amp * comp);

delay_inst1 : ops_dff generic map(AMP_WIDTH) port map(clk, rst, amp_mod_comp_delay(AMP_WIDTH+8-2 downto 7), amp_mod_comp);

mod_data_comp_delay <= signed(cosine_data) * unsigned(amp_mod_comp);

delay_inst2 : ops_dff generic map(PHASE_WIDTH) port map(clk, rst, mod_data_comp_delay(PHASE_WIDTH+AMP_WIDTH-1 downto AMP_WIDTH), mod_data_comp);

m_data_acc <= signed(mod_data_comp)+signed(m_data);
m_rst <= sync or rst;
m_reg_file : reg_file generic map(PHASE_WIDTH) port map(clk, m_rst, mren, reg_ch, m_data_acc, m_data);
			  
with alg_wr select
	f_reg_addr <= alg_ch when '1',
					  reg_ch when others;
			  
f_reg_file : ch_rst_reg_file generic map(PHASE_WIDTH) port map(clk, rst, fren, alg_wr, f_reg_addr, mod_data_comp, f_data);
shifter_inst : shifter port map(f_data, fb, f_data_fl);

with mod_sel select
	poff_in_Norm <= mod_data_comp when "001",
				       m_data_acc when "010",
				       m_data when "011",
				       f_data when "100",
				       f_data_fl when "101",
				       (others => '0') when others;

normalizer_inst : phase_offset_normalizer port map(clk, poff_in_Norm, poff_in);

o_data_acc <= signed(mod_data_comp)+signed(o_data);
o_reg : en_dff generic map(SAMPLE_WIDTH) port map(clk, m_rst, oren, o_data_acc, o_data);
sample_data <= o_data;

op_ctrl_inst : op_ctrl port map(clk, rst, sync, reg_ch, alg_reg_ch, op_en, alg_col, comp_col);

dealy14_inst : xdelay generic map(width=>5, depth=>14) port map(clk, rst, alg_row, comp_row);

alg_rom_inst : alg_rom port map(clk, alg_row, alg_col, comp_row, comp_col, oren, mren, fren, mod_sel, comp);

alg_reg_file_inst : alg_reg_file port map(clk, rst, alg_wr, alg_ch, alg_reg_ch, alg_data, alg_sig);
alg_row <= alg_sig(7 downto 3);
fb_delay <= alg_sig(2 downto 0);
delay_inst3 : ops_dff generic map(3) port map(clk, rst, fb_delay, fb);

-- Phase Accumulator : latency = 1
-- Cosine LUT : latency = 13

-- TODO Multiplier warning
end Behavioral;

