--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 


library IEEE;
use IEEE.STD_LOGIC_1164.all;

package ops_package is
  
  constant FREQ_WIDTH		: integer := 20;
  constant AMP_WIDTH 		: integer := 16; 
  constant PHASE_WIDTH 		: integer := 16;
  constant SAMPLE_WIDTH		: integer := PHASE_WIDTH+4;
  constant NORM_WIDTH      : integer := 10;
  constant NORM_VALUE		: integer := 535;
  constant DAC_WIDTH			: integer := 24;
  constant CTRL_WIDTH		: integer := 8;

	-- Carrier outputs should go to oreg
	type oren32x8 is array (0 to 31, 0 to 7) of std_logic; -- !sync, #6, #5, #4, #3, #2, #1, result
	constant oren_table : oren32x8 :=
				(  ('0', '0', '0', '0', '0', '1', '0', '1'), -- #1
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #2
					('0', '0', '0', '0', '1', '0', '0', '1'), -- #3
					('0', '0', '0', '0', '1', '0', '0', '1'), -- #4
					('0', '0', '0', '1', '0', '1', '0', '1'), -- #5
					('0', '0', '0', '1', '0', '1', '0', '1'), -- #6
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #7
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #8
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #9
					('0', '0', '0', '0', '1', '0', '0', '1'), -- #10
					('0', '0', '0', '0', '1', '0', '0', '1'), -- #11
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #12
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #13
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #14
					('0', '0', '0', '0', '0', '1', '0', '1'), -- #15
					('0', '0', '0', '0', '0', '0', '0', '1'), -- #16
					('0', '0', '0', '0', '0', '0', '0', '1'), -- #17
					('0', '0', '0', '0', '0', '0', '0', '1'), -- #18
					('0', '0', '0', '1', '1', '0', '0', '1'), -- #19
					('0', '0', '0', '0', '1', '0', '1', '1'), -- #20
					('0', '0', '0', '1', '1', '0', '1', '1'), -- #21
					('0', '0', '0', '1', '1', '1', '0', '1'), -- #22
					('0', '0', '0', '1', '1', '0', '1', '1'), -- #23
					('0', '0', '0', '1', '1', '1', '1', '1'), -- #24
					('0', '0', '0', '1', '1', '1', '1', '1'), -- #25
					('0', '0', '0', '0', '1', '0', '1', '1'), -- #26
					('0', '0', '0', '0', '1', '0', '1', '1'), -- #27
					('0', '0', '1', '0', '0', '1', '0', '1'), -- #28
					('0', '0', '0', '1', '0', '1', '1', '1'), -- #29
					('0', '0', '1', '0', '0', '1', '1', '1'), -- #30
					('0', '0', '0', '1', '1', '1', '1', '1'), -- #31
					('0', '0', '1', '1', '1', '1', '1', '1') -- #32
				);

	-- Feedback outputs should go to freg
	type fren32x8 is array (0 to 31, 0 to 7) of std_logic; -- !sync, #6, #5, #4, #3, #2, #1, result
	constant fren_table : fren32x8 :=
				(  ('0', '0', '1', '0', '0', '0', '0', '0'), -- #1
					('0', '0', '0', '0', '0', '0', '1', '0'), -- #2
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #3
					('0', '0', '0', '0', '1', '0', '0', '0'), -- #4
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #5
					('0', '0', '0', '1', '0', '0', '0', '0'), -- #6
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #7
					('0', '0', '0', '0', '1', '0', '0', '0'), -- #8
					('0', '0', '0', '0', '0', '0', '1', '0'), -- #9
					('0', '0', '0', '0', '0', '1', '0', '0'), -- #10
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #11
					('0', '0', '0', '0', '0', '0', '1', '0'), -- #12
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #13
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #14
					('0', '0', '0', '0', '0', '0', '1', '0'), -- #15
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #16
					('0', '0', '0', '0', '0', '0', '1', '0'), -- #17
					('0', '0', '0', '0', '0', '1', '0', '0'), -- #18
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #19
					('0', '0', '0', '0', '0', '1', '0', '0'), -- #20
					('0', '0', '0', '0', '0', '1', '0', '0'), -- #21
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #22
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #23
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #24
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #25
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #26
					('0', '0', '0', '0', '0', '1', '0', '0'), -- #27
					('0', '0', '0', '1', '0', '0', '0', '0'), -- #28
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #29
					('0', '0', '0', '1', '0', '0', '0', '0'), -- #30
					('0', '0', '1', '0', '0', '0', '0', '0'), -- #31
					('0', '0', '1', '0', '0', '0', '0', '0') -- #32
				);

	-- Compensation is only applied to carriers
	type comp32x8 is array (0 to 31, 0 to 7) of std_logic_vector(7 downto 0); -- !sync, #6, #5, #4, #3, #2, #1, result
--	constant comp_table : comp32x8 :=
--				(	(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#1
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#2
--					(x"00", x"00", x"80", x"80", x"40", x"80", x"80", x"40"), --#3
--					(x"00", x"00", x"80", x"80", x"40", x"80", x"80", x"40"), --#4
--					(x"00", x"00", x"80", x"2B", x"80", x"2B", x"80", x"2B"), --#5
--					(x"00", x"00", x"80", x"2B", x"80", x"2B", x"80", x"2B"), --#6
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#7
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#8
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#9
--					(x"00", x"00", x"80", x"80", x"40", x"80", x"80", x"40"), --#10
--					(x"00", x"00", x"80", x"80", x"40", x"80", x"80", x"40"), --#11
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#12
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#13
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#14
--					(x"00", x"00", x"80", x"80", x"80", x"40", x"80", x"40"), --#15
--					(x"00", x"00", x"80", x"80", x"80", x"80", x"80", x"80"), --#16
--					(x"00", x"00", x"80", x"80", x"80", x"80", x"80", x"80"), --#17
--					(x"00", x"00", x"80", x"80", x"80", x"80", x"80", x"80"), --#18
--					(x"00", x"00", x"80", x"2B", x"2B", x"80", x"80", x"2B"), --#19
--					(x"00", x"00", x"80", x"80", x"2B", x"80", x"2B", x"2B"), --#20
--					(x"00", x"00", x"80", x"20", x"20", x"80", x"20", x"20"), --#21
--					(x"00", x"00", x"80", x"20", x"20", x"20", x"80", x"20"), --#22
--					(x"00", x"00", x"80", x"20", x"20", x"80", x"20", x"20"), --#23
--					(x"00", x"00", x"80", x"1A", x"1A", x"1A", x"1A", x"1A"), --#24
--					(x"00", x"00", x"80", x"1A", x"1A", x"1A", x"1A", x"1A"), --#25					
--					(x"00", x"00", x"80", x"80", x"2B", x"80", x"2B", x"2B"), --#26
--					(x"00", x"00", x"80", x"80", x"2B", x"80", x"2B", x"2B"), --#27					
--					(x"00", x"00", x"2B", x"80", x"80", x"2B", x"80", x"2B"), --#28
--					(x"00", x"00", x"80", x"20", x"80", x"20", x"20", x"20"), --#29
--					(x"00", x"00", x"20", x"80", x"80", x"20", x"20", x"20"), --#30
--					(x"00", x"00", x"80", x"1A", x"1A", x"1A", x"1A", x"1A"), --#31
--					(x"00", x"00", x"15", x"15", x"15", x"15", x"15", x"15") --#32				
--				);
-- 80 -> F3
-- 40 -> 79
-- 2B -> 51
-- 20 -> 3D
-- 1A -> 31
-- 15 -> 28
--
--
	constant comp_table : comp32x8 :=
				(	(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#1
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#2
					(x"00", x"00", x"80", x"80", x"79", x"80", x"80", x"79"), --#3
					(x"00", x"00", x"80", x"80", x"79", x"80", x"80", x"79"), --#4
					(x"00", x"00", x"80", x"51", x"80", x"51", x"80", x"51"), --#5
					(x"00", x"00", x"80", x"51", x"80", x"51", x"80", x"51"), --#6
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#7
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#8
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#9
					(x"00", x"00", x"80", x"80", x"79", x"80", x"80", x"79"), --#10
					(x"00", x"00", x"80", x"80", x"79", x"80", x"80", x"79"), --#11
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#12
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#13
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#14
					(x"00", x"00", x"80", x"80", x"80", x"79", x"80", x"79"), --#15
					(x"00", x"00", x"80", x"80", x"80", x"80", x"80", x"F3"), --#16
					(x"00", x"00", x"80", x"80", x"80", x"80", x"80", x"F3"), --#17
					(x"00", x"00", x"80", x"80", x"80", x"80", x"80", x"F3"), --#18
					(x"00", x"00", x"80", x"51", x"51", x"80", x"80", x"51"), --#19
					(x"00", x"00", x"80", x"80", x"51", x"80", x"51", x"51"), --#20
					(x"00", x"00", x"80", x"3D", x"3D", x"80", x"3D", x"3D"), --#21
					(x"00", x"00", x"80", x"3D", x"3D", x"3D", x"80", x"3D"), --#22
					(x"00", x"00", x"80", x"3D", x"3D", x"80", x"3D", x"3D"), --#23
					(x"00", x"00", x"80", x"31", x"31", x"31", x"31", x"31"), --#24
					(x"00", x"00", x"80", x"31", x"31", x"31", x"31", x"31"), --#25					
					(x"00", x"00", x"80", x"80", x"51", x"80", x"51", x"51"), --#26
					(x"00", x"00", x"80", x"80", x"51", x"80", x"51", x"51"), --#27					
					(x"00", x"00", x"51", x"80", x"80", x"51", x"80", x"51"), --#28
					(x"00", x"00", x"80", x"3D", x"80", x"3D", x"3D", x"3D"), --#29
					(x"00", x"00", x"3D", x"80", x"80", x"3D", x"3D", x"3D"), --#30
					(x"00", x"00", x"80", x"31", x"31", x"31", x"31", x"31"), --#31
					(x"00", x"00", x"28", x"28", x"28", x"28", x"28", x"28") --#32				
				);				
	-- Temporary outputs should go to mreg
	type mren32x8 is array (0 to 31, 0 to 7) of std_logic; -- !sync, #6, #5, #4, #3, #2, #1, result
	constant mren_table : mren32x8 :=
				(	('0', '0', '0', '0', '0', '0', '0', '0'), --#1
					('0', '0', '0', '0', '0', '0', '0', '0'), --#2
					('0', '0', '0', '0', '0', '0', '0', '0'), --#3
					('0', '0', '0', '0', '0', '0', '0', '0'), --#4
					('0', '0', '0', '0', '0', '0', '0', '0'), --#5
					('0', '0', '0', '0', '0', '0', '0', '0'), --#6
					('0', '0', '0', '1', '0', '0', '0', '0'), --#7
					('0', '0', '0', '1', '0', '0', '0', '0'), --#8
					('0', '0', '0', '1', '0', '0', '0', '0'), --#9
					('0', '0', '1', '0', '0', '0', '0', '0'), --#10
					('0', '0', '1', '0', '0', '0', '0', '0'), --#11
					('0', '0', '1', '1', '0', '0', '0', '0'), --#12
					('0', '0', '1', '1', '0', '0', '0', '0'), --#13
					('0', '0', '1', '0', '0', '0', '0', '0'), --#14
					('0', '0', '1', '0', '0', '0', '0', '0'), --#15
					('0', '0', '0', '1', '0', '1', '0', '0'), --#16
					('0', '0', '0', '1', '0', '1', '0', '0'), --#17
					('0', '0', '0', '0', '1', '1', '0', '0'), --#18
					('0', '0', '0', '0', '0', '0', '0', '0'), --#19
					('0', '0', '1', '0', '0', '0', '0', '0'), --#20
					('0', '0', '1', '0', '0', '0', '0', '0'), --#21
					('0', '0', '0', '0', '0', '0', '0', '0'), --#22
					('0', '0', '0', '0', '0', '0', '0', '0'), --#23
					('0', '0', '0', '0', '0', '0', '0', '0'), --#24
					('0', '0', '0', '0', '0', '0', '0', '0'), --#25
					('0', '0', '1', '0', '0', '0', '0', '0'), --#26
					('0', '0', '1', '0', '0', '0', '0', '0'), --#27
					('0', '0', '0', '0', '0', '0', '0', '0'), --#28
					('0', '0', '0', '0', '0', '0', '0', '0'), --#29
					('0', '0', '0', '0', '0', '0', '0', '0'), --#30
					('0', '0', '0', '0', '0', '0', '0', '0'), --#31
					('0', '0', '0', '0', '0', '0', '0', '0') --#32
				);

	-- Select appropriate source of modulation
	type mod_sel32x8 is array (0 to 31, 0 to 7) of std_logic_vector(2 downto 0); -- !sync, #6, #5, #4, #3, #2, #1, result
	constant mod_sel_table : mod_Sel32x8 :=
				(	("000", "101", "001", "001", "001", "000", "001", "000"), --#1
					("000", "000", "001", "001", "001", "101", "001", "000"), --#2
					("000", "101", "001", "001", "000", "001", "001", "000"), --#3
					("000", "101", "001", "001", "000", "001", "001", "000"), --#4
					("000", "101", "001", "000", "001", "000", "001", "000"), --#5
					("000", "101", "001", "000", "001", "000", "001", "000"), --#6
					("000", "101", "001", "000", "010", "000", "001", "000"), --#7
					("000", "000", "001", "101", "010", "000", "001", "000"), --#8
					("000", "000", "001", "000", "010", "101", "001", "000"), --#9
					("000", "000", "000", "010", "101", "001", "001", "000"), --#10
					("000", "101", "000", "010", "000", "001", "001", "000"), --#11
					("000", "000", "000", "000", "010", "101", "001", "000"), --#12
					("000", "101", "000", "000", "010", "000", "001", "000"), --#13
					("000", "101", "000", "010", "001", "000", "001", "000"), --#14
					("000", "000", "000", "010", "001", "101", "001", "000"), --#15
					("000", "101", "001", "000", "001", "000", "010", "000"), --#16
					("000", "000", "001", "000", "001", "101", "010", "000"), --#17
					("000", "000", "001", "001", "101", "000", "010", "000"), --#18
					("000", "101", "001", "100", "000", "001", "001", "000"), --#19
					("000", "000", "000", "010", "101", "001", "100", "000"), --#20
					("000", "000", "001", "011", "101", "001", "100", "000"), --#21
					("000", "101", "001", "100", "100", "000", "001", "000"), --#22
					("000", "101", "001", "100", "000", "001", "000", "000"), --#23
					("000", "101", "001", "100", "100", "000", "000", "000"), --#24
					("000", "101", "001", "100", "000", "000", "000", "000"), --#25
					("000", "101", "000", "010", "000", "001", "000", "000"), --#26
					("000", "000", "000", "010", "101", "001", "000", "000"), --#27
					("000", "000", "101", "001", "001", "000", "001", "000"), --#28
					("000", "101", "001", "000", "001", "000", "000", "000"), --#29
					("000", "000", "101", "001", "001", "000", "000", "000"), --#30
					("000", "101", "001", "000", "000", "000", "000", "000"), --#31
					("000", "101", "000", "000", "000", "000", "000", "000") --#32
				);

-- Phase Accumulator
component phase_accu
	port(
		clk : in std_logic;
		rst : in std_logic;
		ch_sclr : in std_logic;
		ch_wr : in std_logic;
		pinc : in std_logic_vector(FREQ_WIDTH-1 downto 0);
		poff : in std_logic_vector(FREQ_WIDTH-1 downto 0);
		pout : out std_logic_vector(FREQ_WIDTH-1 downto 0)
	);
end component;

-- Distributed memory for 96 accumulators
component pa_mem_dist is
	port (
	a: in std_logic_vector(6 downto 0);
	d: in std_logic_vector(19 downto 0);
	dpra: in std_logic_vector(6 downto 0);
	clk: in std_logic;
	we: in std_logic;
	spo: out std_logic_vector(19 downto 0);
	dpo: out std_logic_vector(19 downto 0));
end component;

-- Cosine LUT
component cosine_lut
	port (
	clk: in std_logic;
	phase_in: in std_logic_vector(15 downto 0);
	cosine: out std_logic_vector(15 downto 0));
end component;

-- DFF
component ops_dff
	generic(
		width : integer
	);
	port(
		clk, rst : in std_logic;
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end component;

-- Bit DFF
component b_dff is
	port(
		clk, rst : in std_logic;
		d : in std_logic;
		q : out std_logic
	);
end component;

-- Enable DFF
component en_dff
  generic(
    width : integer
  );
  port(
    clk, rst, en : in std_logic;
    d : in std_logic_vector(width-1 downto 0);
    q : out std_logic_vector(width-1 downto 0)
  );
end component;

-- Negative Edge Enable DFF
component n_en_dff
  generic(
    width : integer
  );
  port(
    clk, rst, en : in std_logic;
    d : in std_logic_vector(width-1 downto 0);
    q : out std_logic_vector(width-1 downto 0)
  );
end component;

-- N-Delay Component
component xdelay
	generic(
		width : integer;
		depth : integer
	);
	port(
		clk : in std_logic;
		rst : in std_logic;
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end component;

-- Register File
component reg_file
	generic(
		width : integer
	);
	port(
		clk, rst, wr: in std_logic;
		addr : in std_logic_vector(3 downto 0); -- 16 channels
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end component;

-- Register File with rst for each channel
component ch_rst_reg_file
	generic(
		width : integer
	);
	port(
		clk, rst, wr, ch_rst: in std_logic;
		addr : in std_logic_vector(3 downto 0); -- 16 channels
		d : in std_logic_vector(width-1 downto 0);
		q : out std_logic_vector(width-1 downto 0)
	);
end component;

-- Shifter
component shifter
	port(
		in_data : in std_logic_vector(PHASE_WIDTH-1 downto 0);
		fb_lvl : in std_logic_vector(2 downto 0);
		out_data : out std_logic_vector(PHASE_WIDTH-1 downto 0)
	);
end component;

--Phase Offset Normalizer
component phase_offset_normalizer
	port(
		clk : in std_logic;
		input : in std_logic_vector(PHASE_WIDTH-1 downto 0);
		output : out std_logic_vector(FREQ_WIDTH-1 downto 0)
	);
end component;

-- Algorithm Register File
component alg_reg_file
	port(
		clk, rst, wr: in std_logic;
		wr_addr : in std_logic_vector(3 downto 0); -- 16 channels
		rd_addr : in std_logic_vector(3 downto 0); -- 16 channels
		d : in std_logic_vector(7 downto 0);
		q : out std_logic_vector(7 downto 0)
	);
end component;

-- OPS controller
component op_ctrl
	port(
		clk : in std_logic;
		rst : in std_logic;
		sync : in std_logic;
		reg_ch : out std_logic_vector(3 downto 0);
		alg_reg_ch : out std_logic_vector(3 downto 0);
		op_en : out std_logic;
		alg_col : out std_logic_vector(2 downto 0);
		comp_col : out std_logic_vector(2 downto 0)
	);
end component;

-- Algorithm rom table
component alg_rom
	port(
		clk : in std_logic;
		alg_row : in std_logic_vector(4 downto 0);
		alg_col : in std_logic_vector(2 downto 0);
		comp_row : in std_logic_vector(4 downto 0);
		comp_col : in std_logic_vector(2 downto 0);
		oren : out std_logic;
		mren : out std_logic;
		fren : out std_logic;
		mod_sel : out std_logic_vector(2 downto 0);
		comp : out std_logic_vector(7 downto 0)
	);
end component;
 
end ops_package;
