----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:16:35 09/20/2011 
-- Design Name: 
-- Module Name:    egs - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
use work.types.all;
use work.egs_package.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity egs is
    Port ( clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           lrck_in : in  STD_LOGIC;
           sync_out : out  STD_LOGIC;
           osc_sync : out  STD_LOGIC;
           freq : out  STD_LOGIC_VECTOR (PHASEWIDTH-1 downto 0);
           amp : out  STD_LOGIC_VECTOR (AMPWIDTH-1 downto 0);
           fsl_data : in  STD_LOGIC_VECTOR (31 downto 0);
           fsl_wen : in  STD_LOGIC;
			  hold : out STD_LOGIC);
end egs;

architecture Behavioral of egs is
	type egs_state_type is (waiting, gl_counting, op_counting, resetting);
	type egs_rec_type is
	record
		state : egs_state_type;
		channel : std_logic_vector(CHBITS-1 downto 0);
		operator : std_logic_vector(OPBITS-1 downto 0);
		writeop : std_logic_vector(0 downto 0);
		writegl : std_logic_vector(0 downto 0);
		sync_out : std_logic;
		old_lrck : std_logic;
		FSL_wen : std_logic;
		FSL_data : std_logic_vector(31 downto 0);
	end record;
	signal r,r_next : egs_rec_type;
	signal lrck_sync : std_logic;
	
	---------------------
	--Memory Read Stage--
	signal mr_op_rd_addr	: std_logic_vector(OPBITS+CHBITS-1 downto 0);
	signal mr_glr_rd_addr	: std_logic_vector(CHBITS-1 downto 0);
	
	-----------------------
	--Memory Decode Stage--
	signal md_op_rd_data : std_logic_vector(EGS_RAMWIDTH-1 downto 0);
	signal md_glr_rd_data : std_logic_vector(EGS_GL_STATEWIDTH-1 downto 0);
	
	signal md_op_wr_addr	: std_logic_vector(OPBITS+CHBITS-1 downto 0);
	signal md_glr_wr_addr	: std_logic_vector(CHBITS-1 downto 0);
	
	
	signal md_r1	: std_logic_vector(EGRWIDTH-1 downto 0);
	signal md_r2	: std_logic_vector(EGRWIDTH-1 downto 0);
	signal md_r3	: std_logic_vector(EGRWIDTH-1 downto 0);
	signal md_r4	: std_logic_vector(EGRWIDTH-1 downto 0);
	signal md_l1	: std_logic_vector(EGLWIDTH-1 downto 0);
	signal md_l2	: std_logic_vector(EGLWIDTH-1 downto 0);
	signal md_l3	: std_logic_vector(EGLWIDTH-1 downto 0);
	signal md_l4	: std_logic_vector(EGLWIDTH-1 downto 0);
	
	signal md_op_eg_stage : std_logic_vector(EGSTAGEWIDTH-1 downto 0);
	signal md_op_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);
	
	signal md_gl_lfodelay : std_logic_vector(LFODELAYWIDTH-1 downto 0);
	signal md_gl_pmodsense : std_logic_vector(PMODWIDTH-1 downto 0);
	signal md_gl_lfospeed : std_logic_vector(LFOSPEEDWIDTH-1 downto 0);
	signal md_gl_lfosync : std_logic;
	signal md_gl_lfowave : std_logic_vector(LFOWAVEWIDTH-1 downto 0);
	signal md_gl_key_on : std_logic;
	
	signal md_glr_eg_stage : std_logic_vector(EGSTAGEWIDTH-1 downto 0);
	signal md_glr_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);
	signal md_glr_lfostate : std_logic;
	signal md_glr_lfophase : std_logic_vector(LFOPHASEWIDTH-1 downto 0);
	signal md_glr_lfowave : std_logic_vector(LFOWAVEWIDTH-1 downto 0);
	signal md_glr_key_on : std_logic;
	signal md_glr_pmodsense : std_logic_vector(PMODWIDTH-1 downto 0);
	signal md_glr_lfodelay : std_logic_vector(LFODELAYWIDTH-1 downto 0);

	signal md_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);
	signal md_eg_stage : std_logic_vector(EGSTAGEWIDTH-1 downto 0);

	signal md_new_key : std_logic;

	signal md_reset : std_logic;
	
	---------------------------
	--Modulation Update Stage--
	
	signal mu_new_key : std_logic;
	
	signal mu_op_rd_data : std_logic_vector(EGS_RAMWIDTH-1 downto 0);
	
	signal mu_glr_wen : std_logic_vector(0 downto 0);
	signal mu_glr_wr_data : std_logic_vector(EGS_GL_STATEWIDTH-1 downto 0);
	signal mu_freq : std_logic_vector(OPFREQWIDTH-1 downto 0);
	signal mu_op_amodsense : std_logic_vector(AMODWIDTH-1 downto 0);
	signal mu_op_outlevel : std_logic_vector(OPAMPWIDTH-1 downto 0);
	signal mu_op_sync_on : std_logic;
	signal mu_op_on : std_logic;
	
	signal mu_eg_stage : std_logic_vector(EGSTAGEWIDTH-1 downto 0);
	signal mu_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);

	signal mu_eg_nextstage : std_logic_vector(EGSTAGEWIDTH-1 downto 0);
	signal mu_eg_nextcount : std_logic_vector(EGCOUNTWIDTH-1 downto 0);

	signal mu_r1	: std_logic_vector(EGCOUNTWIDTH downto 0);
	signal mu_r2	: std_logic_vector(EGCOUNTWIDTH downto 0);
	signal mu_r3	: std_logic_vector(EGCOUNTWIDTH downto 0);
	signal mu_r4	: std_logic_vector(EGCOUNTWIDTH downto 0);
	signal mu_l1	: std_logic_vector(EGLWIDTH-1 downto 0);
	signal mu_l2	: std_logic_vector(EGLWIDTH-1 downto 0);
	signal mu_l3	: std_logic_vector(EGLWIDTH-1 downto 0);
	signal mu_l4	: std_logic_vector(EGLWIDTH-1 downto 0);
	
	signal mu_op_wr_addr : std_logic_vector(OPBITS+CHBITS-1 downto 0);
	signal mu_op_wr_data : std_logic_vector(EGS_RAMWIDTH-1 downto 0);
	signal mu_op_wen		: std_logic_vector(0 downto 0);
	
	signal mu_glr_wr_addr : std_logic_vector(CHBITS-1 downto 0);
	signal mu_glr_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);
	signal mu_glr_lfostate : std_logic;
	signal mu_glr_lfophase : std_logic_vector(LFOPHASEWIDTH-1 downto 0);
	signal mu_glr_lfowave : std_logic_vector(LFOWAVEWIDTH-1 downto 0);
	signal mu_glr_key_on : std_logic;
	signal mu_glr_pmodsense : std_logic_vector(PMODWIDTH-1 downto 0);
	signal mu_glr_lfodelay : std_logic_vector(LFODELAYWIDTH-1 downto 0);
	
	--new values to write
	signal mu_glrw_lfostate : std_logic;
	signal mu_glrw_lfodelay : std_logic_vector(LFODELAYWIDTH-1 downto 0);
	signal mu_glrw_lfophase : std_logic_vector(LFOPHASEWIDTH-1 downto 0);
	signal mu_glrw_pmodsense : std_logic_vector(PMODWIDTH-1 downto 0);
	signal mu_glrw_lfowave : std_logic_vector(LFOWAVEWIDTH-1 downto 0);
	signal mu_glrw_key_on : std_logic;
	
	
	-- signals in r that got renamed for mu
	signal mu_fsl_wen : std_logic;
	signal mu_fsl_data : std_logic_vector(31 downto 0);
	
	signal mu_reset : std_logic;
	signal mu_osc_sync : std_logic;
	--------------------------------
	--Modulation Calculation Stage--
	signal mc_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);
	signal mc_freq : std_logic_vector(OPFREQWIDTH-1 downto 0);
	signal mc_op_amodsense : std_logic_vector(AMODWIDTH-1 downto 0);
	signal mc_op_outlevel : std_logic_vector(OPAMPWIDTH-1 downto 0);
	signal mc_op_sync_on : std_logic;
	signal mc_op_on : std_logic;
	
	signal mc_glr_eg_count : std_logic_vector(EGCOUNTWIDTH-1 downto 0);
	signal mc_glr_lfostate : std_logic;
	signal mc_glr_lfophase : std_logic_vector(LFOPHASEWIDTH-1 downto 0);
	signal mc_glr_lfowave : std_logic_vector(LFOWAVEWIDTH-1 downto 0);
	signal mc_glr_key_on : std_logic;
	signal mc_glr_pmodsense : std_logic_vector(PMODWIDTH-1 downto 0);
	signal mc_glr_shval : std_logic_vector(LFODELAYWIDTH-1 downto 0);
	
	signal mc_sine : std_logic_vector(9 downto 0);
	signal mc_osc_sync : std_logic;
	signal mc_lfoval : std_logic_vector(LFOWIDTH-1 downto 0);	
	
	--------------------------------
	--Modulation Application Stage--
	
	signal ma_osc_sync : std_logic;
	
	signal ma_tlval : std_logic_vector(TLWIDTH-1 downto 0);
	signal ma_amp : std_logic_vector(AMPWIDTH-1 downto 0);
	signal ma_pmultconst : std_logic_vector(23 downto 0);
	signal ma_shamt : std_logic_vector(3 downto 0);
	signal ma_freq : std_logic_vector(PHASEWIDTH-1 downto 0);
	
begin
-- I've divided EGS into 5 different pipeline stages, Memory Read, Memory Decode, Modulation update, Modulation Calculation and Output
	lr_syncer : bit_dff port map(clk,rst,lrck_in,lrck_sync);


-- MR (Memory Read) Stage
memsigs : process(r, rst, lrck_sync, fsl_data, fsl_wen, mu_glr_wen, mu_op_wen, mu_op_wr_addr)
	variable v : egs_rec_type;
begin
	v := r;
	hold <= fsl_wen; -- block all writes as default
	v.channel := fsl_data(FSL_CHOFFSET+CHBITS-1 downto FSL_CHOFFSET);
	if(fsl_data(FSL_OPOFFSET+OPBITS-1 downto FSL_OPOFFSET) = "111") then
		v.operator := (others=>'0');
	else
		v.operator := fsl_data(FSL_OPOFFSET+OPBITS-1 downto FSL_OPOFFSET);
	end if;
	v.writeop := "0";
	v.writegl := "0";
	v.sync_out := '0';
	v.old_lrck := lrck_sync;
	v.FSL_wen := '0';
	v.FSL_data := fsl_data;
	if(rst = '1') then
		v.state := resetting;
		v.writeop := "1";
		v.writegl := "1";
		v.channel := "0000";
		v.operator := "000";
	else
		case(r.state) is
		when waiting => -- done
			if(r.old_lrck = '1' and lrck_sync = '0') then -- transitions must also execute
				v.state := gl_counting;
				v.fsl_wen := '0';
				v.writegl := "1";
				v.operator := "110";
				v.channel := "0000";
			else
				if(r.writeop = "1" or mu_op_wen = "1") then
					v.writeop := "0";
					v.channel := not mu_op_wr_addr(3 downto 0); -- protects from reading and writing the same address
				else
					v.writeop(0) := fsl_wen; 
					v.fsl_wen := fsl_wen;
					hold <= '0'; -- we're free to write!
				end if;
			end if;
		when gl_counting =>
			if(r.channel = "1111") then
				v.state := op_counting;
				v.operator := "000";
				v.channel := "0000";
				v.writegl := "0";
				v.writeop := "1";
				v.sync_out := '1';
			else
				v.operator := "110";
				v.channel := r.channel + 1;
				v.writegl := "1";
			end if;
		when op_counting =>
			v.writeop := "1";
			if(r.channel = "1111") then
				if(r.operator = "101") then
					v.state := waiting;
					v.writeop := "0";
				else
					v.operator := r.operator + 1;
					v.channel := "0000";
				end if;
			else
				v.channel := r.channel + 1;
				v.operator := r.operator;
			end if;
		when resetting =>
			v.channel := r.channel + 1;
			v.operator := r.operator;
			v.writeop := "1";
			v.writegl := "1";
			if(r.channel = "1111") then
				if(r.operator = "110") then -- erase globals as well
					v.state := waiting;
					v.writeop := "0";
					v.writegl := "0";
				else
					v.operator := r.operator + 1;
				end if;
			end if;
		end case;
	end if;
	r_next <= v;
end process;

-- op & parameter ram
mr_op_rd_addr <= r_next.operator & r_next.channel;
mr_glr_rd_addr <= r_next.channel;


--Pipe MR/MD
mrmd_op_addr_delay : dff generic map(CHBITS+OPBITS) port map(clk,rst,mr_op_rd_addr,md_op_wr_addr);
mrmd_gl_addr_delay : dff generic map(CHBITS) port map(clk,rst,mr_glr_rd_addr,md_glr_wr_addr);

op_mem : egsram
		port map (
			clka => clk,
			wea => mu_op_wen,
			addra => mu_op_wr_addr,
			dina => mu_op_wr_data,
			clkb => clk,
			addrb => mr_op_rd_addr,
			doutb => md_op_rd_data);

gl_mem : globram
		port map (
			clka => clk,
			wea => mu_glr_wen,
			addra => mu_glr_wr_addr,
			dina => mu_glr_wr_data,
			clkb => clk,
			addrb => mr_glr_rd_addr,
			doutb => md_glr_rd_data);

sync : process(clk)
begin
	if(clk'event and clk = '1') then
		r <= r_next;
	end if;
end process;

--MD Stage

-- Divide read line into signals
md_r1 <= md_op_rd_data(OP_R1OFFSET+EGRWIDTH-1 downto OP_R1OFFSET);
md_r2 <= md_op_rd_data(OP_R2OFFSET+EGRWIDTH-1 downto OP_R2OFFSET);
md_r3 <= md_op_rd_data(OP_R3OFFSET+EGRWIDTH-1 downto OP_R3OFFSET);
md_r4 <= md_op_rd_data(OP_R4OFFSET+EGRWIDTH-1 downto OP_R4OFFSET);
md_l1 <= md_op_rd_data(OP_L1OFFSET+EGLWIDTH-1 downto OP_L1OFFSET);
md_l2 <= md_op_rd_data(OP_L2OFFSET+EGLWIDTH-1 downto OP_L2OFFSET);
md_l3 <= md_op_rd_data(OP_L3OFFSET+EGLWIDTH-1 downto OP_L3OFFSET);
md_l4 <= md_op_rd_data(OP_L4OFFSET+EGLWIDTH-1 downto OP_L4OFFSET);
	
md_op_eg_stage <= md_op_rd_data(OP_EGSTAGEOFFSET+EGSTAGEWIDTH-1 downto OP_EGSTAGEOFFSET);
md_op_eg_count <= md_op_rd_data(OP_EGCOUNTOFFSET+EGCOUNTWIDTH-1 downto OP_EGCOUNTOFFSET);

md_gl_lfodelay <= md_op_rd_data(GL_LFODELAYOFFSET+LFODELAYWIDTH-1 downto GL_LFODELAYOFFSET);
md_gl_pmodsense <= md_op_rd_data(GL_PMODOFFSET+PMODWIDTH-1 downto GL_PMODOFFSET);
md_gl_lfospeed <= md_op_rd_data(GL_LFOSPEEDOFFSET+LFOSPEEDWIDTH-1 downto GL_LFOSPEEDOFFSET);
md_gl_lfosync <= md_op_rd_data(GL_LFOSYNCOFFSET);
md_gl_lfowave <= md_op_rd_data(GL_LFOWAVEOFFSET+LFOWAVEWIDTH-1 downto GL_LFOWAVEOFFSET);
md_gl_key_on <= md_op_rd_data(GL_KEYONOFFSET);

md_glr_eg_stage <= md_glr_rd_data(GLR_EGSTAGEOFFSET+EGSTAGEWIDTH-1 downto GLR_EGSTAGEOFFSET);
md_glr_eg_count <= md_glr_rd_data(GLR_EGCOUNTOFFSET+EGCOUNTWIDTH-1 downto GLR_EGCOUNTOFFSET);
md_glr_lfostate <= md_glr_rd_data(GLR_LFOSTATEOFFSET);
md_glr_lfophase <= md_glr_rd_data(GLR_LFOPHASEOFFSET+LFOPHASEWIDTH-1 downto GLR_LFOPHASEOFFSET);
md_glr_lfowave <= md_glr_rd_data(GLR_LFOWAVEOFFSET+LFOWAVEWIDTH-1 downto GLR_LFOWAVEOFFSET);
md_glr_key_on <= md_glr_rd_data(GLR_KEYONOFFSET);
md_glr_pmodsense <= md_glr_rd_data(GLR_PMODOFFSET+PMODWIDTH-1 downto GLR_PMODOFFSET);
md_glr_lfodelay <= md_glr_rd_data(GLR_LFODELAYCOUNTOFFSET+LFODELAYWIDTH-1 downto GLR_LFODELAYCOUNTOFFSET);
	
with r.state select
	md_eg_stage <= md_op_eg_stage when op_counting,
	md_glr_eg_stage when others;

with r.state select
	md_eg_count <= md_op_eg_count when op_counting,
	md_glr_eg_count when others;

md_new_key <= md_gl_key_on and (not (md_eg_stage(0) or md_eg_stage(1)));

with r.state select
md_reset <= '1' when resetting,
'0' when others;

mdmu_reset_delay : bit_dff port map(clk,rst,md_reset,mu_reset);

-- PIPE MD/MU
mdmu_rc_inst : rate_converter port map(clk,rst,md_r1,md_r2,md_r3,md_r4,mu_r1,mu_r2,mu_r3,mu_r4);
mdmu_l1_delay : dff generic map(EGLWIDTH) port map(clk,rst,md_l1,mu_l1);
mdmu_l2_delay : dff generic map(EGLWIDTH) port map(clk,rst,md_l2,mu_l2);
mdmu_l3_delay : dff generic map(EGLWIDTH) port map(clk,rst,md_l3,mu_l3);
mdmu_l4_delay : dff generic map(EGLWIDTH) port map(clk,rst,md_l4,mu_l4);

mdmu_op_wen_delay : dff generic map(1) port map(clk,rst,r.writeop,mu_op_wen);
mdmu_op_addr_delay : dff generic map(CHBITS+OPBITS) port map(clk,rst,md_op_wr_addr,mu_op_wr_addr);
mdmu_op_rd_data_delay : dff generic map(EGS_RAMWIDTH) port map(clk,rst,md_op_rd_data,mu_op_rd_data);


mdmu_glr_wen_delay : dff generic map(1) port map(clk,rst,r.writegl,mu_glr_wen);
mdmu_glr_addr_delay : dff generic map(CHBITS) port map(clk,rst,md_glr_wr_addr,mu_glr_wr_addr);


mdmu_eg_stage_delay : dff generic map(EGSTAGEWIDTH) port map(clk,rst,md_eg_stage,mu_eg_stage);
mdmu_eg_count_delay : dff generic map(EGCOUNTWIDTH) port map(clk,rst,md_eg_count,mu_eg_count);

mdmu_glrw_pmodsense_delay : dff generic map(PMODWIDTH) port map(clk,rst,md_gl_pmodsense,mu_glrw_pmodsense);
mdmu_glrw_lfowave_delay : dff generic map(LFOWAVEWIDTH) port map(clk,rst,md_gl_lfowave,mu_glrw_lfowave);
mdmu_glrw_key_on_delay : bit_dff port map(clk,rst,md_gl_key_on,mu_glrw_key_on);


mdmu_glr_eg_count_delay : dff generic map(EGCOUNTWIDTH) port map(clk,rst,md_glr_eg_count,mu_glr_eg_count);
mdmu_glr_lfostate_delay : bit_dff port map(clk,rst,md_glr_lfostate,mu_glr_lfostate);
mdmu_glr_lfophase_delay : dff generic map(LFOPHASEWIDTH) port map(clk,rst,md_glr_lfophase,mu_glr_lfophase);
mdmu_glr_lfowave_delay : dff generic map(LFOWAVEWIDTH) port map(clk,rst,md_glr_lfowave,mu_glr_lfowave);
mdmu_glr_key_on_delay : bit_dff port map(clk,rst,md_glr_key_on,mu_glr_key_on);
mdmu_glr_pmodsense_delay : dff generic map(PMODWIDTH) port map(clk,rst,md_glr_pmodsense,mu_glr_pmodsense);
mdmu_glr_lfodelay_delay : dff generic map(LFODELAYWIDTH) port map(clk,rst,md_glr_lfodelay,mu_glr_lfodelay);

mdmu_fsl_wen_delay : bit_dff port map(clk,rst,r.FSL_wen,mu_fsl_wen);
mdmu_fsl_data_delay : dff generic map(32) port map(clk,rst,r.FSL_data,mu_fsl_data);

lfo_pacc_inst : lfo_phaseacc port map(clk,rst,md_gl_lfospeed,md_gl_lfosync,md_new_key,md_gl_lfodelay,md_glr_lfostate,md_glr_lfodelay,md_glr_lfophase,mu_glrw_lfostate,mu_glrw_lfodelay,mu_glrw_lfophase);
mdmu_new_key_delay : bit_dff port map(clk,rst,md_new_key,mu_new_key);

--Modulation Update Stage
eg_inst : eg port map(
mu_r1,
mu_r2,
mu_r3,
mu_r4,
mu_l1,
mu_l2,
mu_l3,
mu_l4,
mu_eg_stage, 
mu_eg_count,
mu_glr_key_on,
mu_eg_nextstage,
mu_eg_nextcount);

with mu_reset select
mu_glr_wr_data <= (others=>'0') when '1',
mu_eg_nextstage & mu_eg_nextcount & mu_glrw_lfostate & mu_glrw_lfophase & mu_glrw_lfowave & mu_glrw_key_on & mu_glrw_pmodsense & mu_glrw_lfodelay when others;


--this acts as a mux for op_wr_data line. 
gen_op_wr_data : process(mu_op_rd_data, mu_fsl_wen,mu_fsl_data(FSL_RATEBITOFFSET), mu_fsl_data(FSL_PAROFFSET+PARBITS-1 downto FSL_PAROFFSET), mu_reset, mu_eg_nextstage, mu_eg_nextcount)
begin
	mu_op_wr_data <= mu_op_rd_data;
	if(mu_fsl_wen = '1') then
		if(mu_fsl_data(FSL_RATEBITOFFSET) = '1') then
			case(mu_fsl_data(FSL_STAGEOFFSET+1 downto FSL_STAGEOFFSET)) is
			when "00" => mu_op_wr_data(OP_R1OFFSET+EGS_EGRATEWIDTH-1 downto OP_R1OFFSET) <= mu_fsl_data(EGS_EGRATEWIDTH-1 downto 0);
			when "01" => mu_op_wr_data(OP_R2OFFSET+EGS_EGRATEWIDTH-1 downto OP_R2OFFSET) <= mu_fsl_data(EGS_EGRATEWIDTH-1 downto 0);
			when "10" => mu_op_wr_data(OP_R3OFFSET+EGS_EGRATEWIDTH-1 downto OP_R3OFFSET) <= mu_fsl_data(EGS_EGRATEWIDTH-1 downto 0);
			when "11" => mu_op_wr_data(OP_R4OFFSET+EGS_EGRATEWIDTH-1 downto OP_R4OFFSET) <= mu_fsl_data(EGS_EGRATEWIDTH-1 downto 0);
			when others=> 
			end case;
		else
			case(mu_fsl_data(FSL_PAROFFSET+PARBITS-1 downto FSL_PAROFFSET)) is
			when "000" => mu_op_wr_data(EGS_PAR0OFFSET+EGS_PAR0WIDTH-1 downto EGS_PAR0OFFSET) <= mu_fsl_data(EGS_PAR0WIDTH-1 downto 0);
			when "001" => mu_op_wr_data(EGS_PAR1OFFSET+EGS_PAR1WIDTH-1 downto EGS_PAR1OFFSET) <= mu_fsl_data(EGS_PAR1WIDTH-1 downto 0);
			when "010" => mu_op_wr_data(EGS_PAR2OFFSET+EGS_PAR2WIDTH-1 downto EGS_PAR2OFFSET) <= mu_fsl_data(EGS_PAR2WIDTH-1 downto 0);
			when "011" => mu_op_wr_data(EGS_PAR3OFFSET+EGS_PAR3WIDTH-1 downto EGS_PAR3OFFSET) <= mu_fsl_data(EGS_PAR3WIDTH-1 downto 0);
			when "100" => mu_op_wr_data(EGS_PAR4OFFSET+EGS_PAR4WIDTH-1 downto EGS_PAR4OFFSET) <= mu_fsl_data(EGS_PAR4WIDTH-1 downto 0);
			when "101" => mu_op_wr_data(EGS_PAR5OFFSET+EGS_PAR5WIDTH-1 downto EGS_PAR5OFFSET) <= mu_fsl_data(EGS_PAR5WIDTH-1 downto 0);
			when "110" => mu_op_wr_data(EGS_PAR6OFFSET+EGS_PAR6WIDTH-1 downto EGS_PAR6OFFSET) <= mu_fsl_data(EGS_PAR6WIDTH-1 downto 0);
			when "111" => mu_op_wr_data(EGS_PAR7OFFSET+EGS_PAR7WIDTH-1 downto EGS_PAR7OFFSET) <= mu_fsl_data(EGS_PAR7WIDTH-1 downto 0);
			when others=> 
			end case;
		end if;
	else
		mu_op_wr_data(EGS_OP_STATEWIDTH-1 downto 0) <= mu_eg_nextstage & mu_eg_nextcount;
	end if;
	if(mu_reset = '1') then
		mu_op_wr_data <= (others=>'0');
	end if;
end process;	

mu_freq <= mu_op_rd_data(OP_FREQOFFSET+OPFREQWIDTH-1 downto OP_FREQOFFSET);
mu_op_amodsense <= mu_op_rd_data(OP_AMODOFFSET+AMODWIDTH-1 downto OP_AMODOFFSET);
mu_op_outlevel <= mu_op_rd_data(OP_OUTLEVELOFFSET+OPAMPWIDTH-1 downto OP_OUTLEVELOFFSET);
mu_op_sync_on <= mu_op_rd_data(OP_SYNCONOFFSET);
mu_op_on <= mu_op_rd_data(OP_ONOFFSET);

mu_osc_sync <= mu_new_key and mu_op_sync_on;

--MU/MC Pipeline Stage
mumc_freq_delay : dff generic map(OPFREQWIDTH) port map(clk,rst,mu_freq,mc_freq);
mumc_amodsense_delay : dff generic map(AMODWIDTH) port map(clk,rst,mu_op_amodsense,mc_op_amodsense);
mumc_outlevel_delay : dff generic map(OPAMPWIDTH) port map(clk,rst,mu_op_outlevel,mc_op_outlevel);
mumc_op_syncon_delay : bit_dff port map(clk,rst,mu_op_sync_on,mc_op_sync_on);
mumc_md_op_on_delay : bit_dff port map(clk,rst,mu_op_on,mc_op_on);

mumc_eg_count_delay : dff generic map(EGCOUNTWIDTH) port map(clk,rst,mu_eg_count,mc_eg_count);
mumc_glr_eg_count_delay : dff generic map(EGCOUNTWIDTH) port map(clk,rst,mu_glr_eg_count,mc_glr_eg_count);
mumc_glr_lfostate_delay : bit_dff port map(clk,rst,mu_glr_lfostate,mc_glr_lfostate);
mumc_glr_lfophase_delay : dff generic map(LFOPHASEWIDTH) port map(clk,rst,mu_glr_lfophase,mc_glr_lfophase);
mumc_glr_lfowave_delay : dff generic map(LFOWAVEWIDTH) port map(clk,rst,mu_glr_lfowave,mc_glr_lfowave);
mumc_glr_key_on_delay : bit_dff port map(clk,rst,mu_glr_key_on,mc_glr_key_on);
mumc_glr_pmodsense_delay : dff generic map(PMODWIDTH) port map(clk,rst,mu_glr_pmodsense,mc_glr_pmodsense);

mumc_glr_lfodelay_delay : dff generic map(LFODELAYWIDTH) port map(clk,rst,mu_glr_lfodelay,mc_glr_shval);

s_lut_inst : sine_lut port map(clk,mu_glr_lfophase(LFOPHASEWIDTH-1 downto LFOPHASEWIDTH-16),mc_sine);

mumc_osc_sync_delay : bit_dff port map(clk,rst,mu_osc_sync,mc_osc_sync);


--Modulation Calculation Stage--
lfo_inst : lfo_wavegen port map(mc_glr_lfophase,mc_glr_lfowave,mc_glr_shval,mc_sine,mc_glr_lfostate,mc_lfoval);

--MC/MA pipe
mco_osc_sync_delay : bit_dff port map(clk,rst,mc_osc_sync,ma_osc_sync);
acalc_inst : ampcalc port map(clk,rst,mc_op_on,mc_op_outlevel,mc_op_amodsense,mc_eg_count,mc_lfoval,ma_tlval);
mcma_freq_delay : dff generic map(PHASEWIDTH) port map(clk,rst,mc_freq,ma_freq);
pshiftcinst : pshiftcalc port map(clk,rst,mc_glr_pmodsense,mc_glr_eg_count,mc_lfoval,ma_pmultconst, ma_shamt);

--Modulation Application Stage--
i_gen : indexgen generic map(AMPWIDTH) port map(ma_tlval,ma_amp);

--MA/OUT pipe
ampdff : dff generic map(AMPWIDTH) port map(clk,rst,ma_amp,amp);
syncoutdff : bit_dff port map(clk,rst,r.sync_out,sync_out);
fcalc_inst : freqcalc port map(clk,rst,ma_freq,ma_pmultconst,ma_shamt,freq);
osc_sync_delay : bit_dff port map(clk,rst,ma_osc_sync,osc_sync);

end Behavioral;