library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
entity rate_converter is
	generic(
		INWIDTH	: integer := 22;
		OUTWIDTH : integer := 33;
		SBITS		: integer := 4
	);
	port(
		clk,rst : in std_logic;
		r1_in : in std_logic_vector(INWIDTH-1 downto 0);
		r2_in : in std_logic_vector(INWIDTH-1 downto 0);
		r3_in : in std_logic_vector(INWIDTH-1 downto 0);
		r4_in : in std_logic_vector(INWIDTH-1 downto 0);
		r1_out : out std_logic_vector(OUTWIDTH-1 downto 0);
		r2_out : out std_logic_vector(OUTWIDTH-1 downto 0);
		r3_out : out std_logic_vector(OUTWIDTH-1 downto 0);
		r4_out : out std_logic_vector(OUTWIDTH-1 downto 0)
	);
end rate_converter;
  
architecture implementation of rate_converter is
	signal r1_mant : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r2_mant : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r3_mant : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r4_mant : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r1_out_next : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r2_out_next : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r3_out_next : std_logic_vector(OUTWIDTH-1 downto 0);
	signal r4_out_next : std_logic_vector(OUTWIDTH-1 downto 0);
begin

r1_mant(OUTWIDTH-1 downto INWIDTH-SBITS) <= (others=>'0');
r1_mant(INWIDTH-SBITS-1 downto 0) <= r1_in(INWIDTH-SBITS-1 downto 0);
r2_mant(OUTWIDTH-1 downto INWIDTH-SBITS) <= (others=>'0');
r2_mant(INWIDTH-SBITS-1 downto 0) <= r2_in(INWIDTH-SBITS-1 downto 0);
r3_mant(OUTWIDTH-1 downto INWIDTH-SBITS) <= (others=>'0');
r3_mant(INWIDTH-SBITS-1 downto 0) <= r3_in(INWIDTH-SBITS-1 downto 0);
r4_mant(OUTWIDTH-1 downto INWIDTH-SBITS) <= (others=>'0');
r4_mant(INWIDTH-SBITS-1 downto 0) <= r4_in(INWIDTH-SBITS-1 downto 0);

r1_out_next <= std_logic_vector(shift_left(unsigned(r1_mant),to_integer(unsigned(r1_in(INWIDTH-1 downto INWIDTH-SBITS)))));
r2_out_next <= std_logic_vector(shift_left(unsigned(r2_mant),to_integer(unsigned(r2_in(INWIDTH-1 downto INWIDTH-SBITS)))));
r3_out_next <= std_logic_vector(shift_left(unsigned(r3_mant),to_integer(unsigned(r3_in(INWIDTH-1 downto INWIDTH-SBITS)))));
r4_out_next <= std_logic_vector(shift_left(unsigned(r4_mant),to_integer(unsigned(r4_in(INWIDTH-1 downto INWIDTH-SBITS)))));

sync : process(clk,rst)
begin
	if(clk'event and clk = '1') then
		if(rst = '1') then
			r1_out <= (others=>'0');
			r2_out <= (others=>'0');
			r3_out <= (others=>'0');
			r4_out <= (others=>'0');
		else
			r1_out <= r1_out_next;
			r2_out <= r2_out_next;
			r3_out <= r3_out_next;
			r4_out <= r4_out_next;
		end if;
	end if;
end process;


end implementation;

---------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;
	
entity eg is
	generic(
		LWIDTH	: integer := 12;
		EGWIDTH	: integer := 32
	);
	port(
		r1_in : in std_logic_vector(EGWIDTH downto 0);
		r2_in : in std_logic_vector(EGWIDTH downto 0);
		r3_in : in std_logic_vector(EGWIDTH downto 0);
		r4_in : in std_logic_vector(EGWIDTH downto 0);
		l1_in : in std_logic_vector(LWIDTH-1 downto 0);
		l2_in : in std_logic_vector(LWIDTH-1 downto 0);
		l3_in : in std_logic_vector(LWIDTH-1 downto 0);
		l4_in : in std_logic_vector(LWIDTH-1 downto 0);
		cur_stage : in std_logic_vector(1 downto 0);
		cur_count : in std_logic_vector(EGWIDTH-1 downto 0);
		key_on	: in std_logic;
		next_stage : out std_logic_vector(1 downto 0);
		next_count : out std_logic_vector(EGWIDTH-1 downto 0)
	);
end eg;
  
architecture implementation of eg is
begin

comb : process(r1_in, r2_in, r3_in, r4_in, l1_in, l2_in, l3_in, l4_in, cur_stage, cur_count, key_on)
	variable smallr1,smallr2,smallr3,smallr4 : std_logic_vector(LWIDTH downto 0);
	variable r1,r2,r3,r4 : std_logic_vector(EGWIDTH-1 downto 0);
	variable l1,l2,l3,l4 : std_logic_vector(EGWIDTH-1 downto 0);
	variable s1,s2,s3,s4 : std_logic;
	variable smallcount : std_logic_vector(LWIDTH-1 downto 0);
	
begin
	next_stage <= cur_stage;
--	smallr1 := r1_in(EGWIDTH downto EGWIDTH-LWIDTH); -- upper bits of rates. Needed to speed up compares...
--	smallr2 := r2_in(EGWIDTH downto EGWIDTH-LWIDTH);
--	smallr3 := r3_in(EGWIDTH downto EGWIDTH-LWIDTH);
--	smallr4 := r4_in(EGWIDTH downto EGWIDTH-LWIDTH);
	smallcount := cur_count(EGWIDTH-1 downto EGWIDTH-LWIDTH); -- upper bits of count
	--smalll1 := '0' & l1_in;
	
	r1 := r1_in(EGWIDTH-1 downto 0);
	r2 := r2_in(EGWIDTH-1 downto 0);
	r3 := r3_in(EGWIDTH-1 downto 0);
	r4 := r4_in(EGWIDTH-1 downto 0);
	l1 := l1_in & x"00000";
	l2 := l2_in & x"00000";
	l3 := l3_in & x"00000";
	l4 := l4_in & x"00000";
	
	-- It's important to keep in mind that level has trailing zeros! Use this to calculate stuff!
	-- The s's are used to determine wheter addition or subtraction is needed in the different 
	-- EG stages.
	if(smallcount < l1_in) then
		s1 := '0';
	else
		s1 := '1'; -- 1 is for subtract
	end if;
	if(smallcount < l2_in) then
		s2 := '0';
	else
		s2 := '1';
	end if;
	if(smallcount < l3_in) then
		s3 := '0'; 
	else
		s3 := '1';
	end if;
	if(smallcount < l4_in) then
		s4 := '0';
	else
		s4 := '1';
	end if;
	
	case cur_stage is
	when "00" => -- Moore states. Even if key is pressed we need to switch state before adding
		-- moving towards l4 with r4
		next_stage <= '0' & key_on;
		if(s4 = '0') then -- add
			if((l4 - cur_count) <= r4) then -- check if overflow when adding
				next_count <= l4;
			else
				next_count <= cur_count + r4;
			end if;
		else -- subtract
			if((cur_count - l4) <= r4) then
				next_count <= l4;
			else
				next_count <= cur_count - r4;
			end if;
		end if;
	when "01" =>
		-- moving towards l1 with r1
		if(s1 = '0') then -- add
			if((l1 - cur_count) <= r1) then -- check if overflow when adding
				next_count <= l1;
				next_stage <= "11";
			else
				next_count <= cur_count + r1;
			end if;
		else -- subtract
			if((cur_count - l1) <= r1) then
				next_count <= l1;
				next_stage <= "11";
			else
				next_count <= cur_count - r1;
			end if;
		end if;
	when "11" =>
		-- moving towards l2 with r2
		if(s2 = '0') then -- add
			if((l2 - cur_count) <= r2) then -- check if overflow when adding
				next_count <= l2;
				next_stage <= "10";
			else
				next_count <= cur_count + r2;
			end if;
		else -- subtract
			if((cur_count - l2) <= r2) then
				next_count <= l2;
				next_stage <= "10";
			else
				next_count <= cur_count - r2;
			end if;
		end if;
	when "10" =>
		-- moving towards l3 with r3. Stay in this stage!
		if(s3 = '0') then -- add
			if((l3 - cur_count) <= r3) then -- check if overflow when adding
				next_count <= l3;
			else
				next_count <= cur_count + r3;
			end if;
		else -- subtract
			if((cur_count - l3) <= r3) then
				next_count <= l3;
			else
				next_count <= cur_count - r3;
			end if;
		end if;
	when others => -- amp level would be held at l3, since we don't change state until a key is depressed
	end case;
	if(key_on = '0') then -- the envelope should immediately go to l4 at r4 if a key is not pressed
		next_stage <= "00";
	end if;
end process;

end implementation;

---------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
	
-- Ok, so we know that the output of the dx7 is a value expressed in half periods as
-- 2 ^ ((33/16) - (TL/8)), where TL is a value between 0 and 127. The key idea in this
-- design is to create this output by taking 2^(32/16) * 2^((1/16) - (TL mod 8)) * 2^(-TL/8),
-- which is simplified further to 4 * C(8 entry table) >> (TL/8)
entity indexgen is
	generic(
		width : integer := 14
	);
	port(
		tlval : in std_logic_vector(8 downto 0);
		output : out std_logic_vector(width-1 downto 0)
	);
end indexgen;
  
architecture implementation of indexgen is
	constant constwidth : integer := 27; -- 27 bits max width. Const in 3.24 format
	signal int : std_logic_vector(3 downto 0);
	signal man : std_logic_vector(4 downto 0);
	signal const : std_logic_vector(constwidth-1 downto 0);
begin

int <= tlval(8 downto 5);
man <= tlval(4 downto 0);

comb : process(man)
begin
	case(man) is
		when "00000" => const <= O"4" & X"2D561B";
		when "00001" => const <= O"4" & X"166C35";
		when "00010" => const <= O"4" & X"000000";
		when "00011" => const <= O"3" & X"EA0ECB";
		when "00100" => const <= O"3" & X"D495F4";
		when "00101" => const <= O"3" & X"BF92E6";
		when "00110" => const <= O"3" & X"AB031C";
		when "00111" => const <= O"3" & X"96E41C";
		when "01000" => const <= O"3" & X"83337C";
		when "01001" => const <= O"3" & X"6FEEDE";
		when "01010" => const <= O"3" & X"5D13F3";
		when "01011" => const <= O"3" & X"4AA076";
		when "01100" => const <= O"3" & X"389230";
		when "01101" => const <= O"3" & X"26E6F6";
		when "01110" => const <= O"3" & X"159CA8";
		when "01111" => const <= O"3" & X"04B133";
		when "10000" => const <= O"2" & X"F4228E";
		when "10001" => const <= O"2" & X"E3EEBD";
		when "10010" => const <= O"2" & X"D413CD";
		when "10011" => const <= O"2" & X"C48FD6";
		when "10100" => const <= O"2" & X"B560FC";
		when "10101" => const <= O"2" & X"A6856B";
		when "10110" => const <= O"2" & X"97FB5B";
		when "10111" => const <= O"2" & X"89C10C";
		when "11000" => const <= O"2" & X"7BD4CA";
		when "11001" => const <= O"2" & X"6E34E7";
		when "11010" => const <= O"2" & X"60DFC1";
		when "11011" => const <= O"2" & X"53D3BF";
		when "11100" => const <= O"2" & X"470F4E";
		when "11101" => const <= O"2" & X"3A90E6";
		when "11110" => const <= O"2" & X"2E5708";
		when "11111" => const <= O"2" & X"22603A";
		
		
		
		when others => const <= (others=>'0');	
	end case;
end process;
output <= std_logic_vector(shift_right(unsigned(const(constwidth-1 downto constwidth-width)),to_integer(unsigned(int))));
end implementation;


----------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;
--use IEEE.numeric_std.all;
	
entity lfo_phaseacc is
	generic(
		regwidth : integer := 20;
		incwidth : integer := 12;
		delaywidth : integer := 20
	);
	port(
		clk,rst : in std_logic;
		pinc : in std_logic_vector(incwidth-1 downto 0);
		sync_on : in std_logic;
		new_key : in std_logic;
		delay : in std_logic_vector(delaywidth-1 downto 0); -- given in cycles
		dstate_in : in std_logic;
		delayreg_in : in std_logic_vector(delaywidth-1 downto 0);
		phasereg_in : in std_logic_vector(regwidth-1 downto 0);
		dstate_out : out std_logic;
		delayreg_out : out std_logic_vector(delaywidth-1 downto 0);
		phasereg_out : out std_logic_vector(regwidth-1 downto 0)
	);
end lfo_phaseacc;
  
architecture implementation of lfo_phaseacc is
	signal dstate_next : std_logic; -- '0' = in delay state
	signal delayreg_next : std_logic_vector(delaywidth-1 downto 0);
	signal phasereg_next : std_logic_vector(regwidth-1 downto 0);
begin
comb : process(pinc,sync_on,new_key,delay,dstate_in,delayreg_in,phasereg_in)
begin
	phasereg_next <= phasereg_in + pinc;
	if(new_key = '1') then
		delayreg_next <= delay;
		dstate_next <= delay(delaywidth-1);
		if(sync_on = '1') then
			phasereg_next <= (others=>'0');
		end if;
	else
		if(dstate_in = '0') then
			delayreg_next <= delayreg_in + 1;
		else 	-- act as an lfsr for pseudo random number (S & H waveform) 
				--generation after delay
			if(pinc > (not phasereg_in)) then
				--fibonacci polynomial to generate a pseudo-random sequence of 65535 values.
				delayreg_next <= '1' & (delayreg_in(delaywidth-18) xor delayreg_in(delaywidth-16) xor delayreg_in(delaywidth-15) xor delayreg_in(delaywidth-13)) & delayreg_in(delaywidth-2 downto 1);
			else
				delayreg_next <= delayreg_in;				
			end if;
		end if;
		dstate_next <= delayreg_in(delaywidth-1);
	end if;
	
end process;

sync : process(clk,rst)
begin
	if(clk'event and clk = '1') then
		if(rst = '1') then
			dstate_out <= '0';
			delayreg_out <= (others=>'0');
			phasereg_out <= (others=>'0');
		else
			dstate_out <= dstate_next;
			delayreg_out <= delayreg_next;
			phasereg_out <= phasereg_next;
		end if;
	end if;
end process;
end implementation;


----------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
	
entity lfo_wavegen is
	generic(
		phasewidth : integer := 20;
		outwidth : integer := 12
	);
	port(
		phase : in std_logic_vector(phasewidth-1 downto 0);
		wave : in std_logic_vector(2 downto 0);
		shval : in std_logic_vector(phasewidth-1 downto 0);
		sineval : in std_logic_vector(9 downto 0);
		dstate : in std_logic;
		value : out std_logic_vector(outwidth-1 downto 0)
	);
end lfo_wavegen;
architecture implementation of lfo_wavegen is
	
begin
produce_value : process(phase,wave,shval,sineval,dstate)
begin
	if(dstate = '0') then
		value <= (others=>'0');
	else
		case(wave) is
		when "000" => -- triangle
		if(phase(phasewidth-1) = '0') then
			value <= not phase(phasewidth-2) & phase(phasewidth-3 downto phasewidth-outwidth-1);
		else
			value <= phase(phasewidth-2) & not phase(phasewidth-3 downto phasewidth-outwidth-1);
		end if;
		when "001" => -- saw down
			value <= phase(phasewidth-1) & not phase(phasewidth-2 downto phasewidth-outwidth);
		when "010" => -- saw up
			value <= not phase(phasewidth-1) & phase(phasewidth-2 downto phasewidth-outwidth);
		when "011" => -- square
		if(phase(phasewidth-1) = '1') then
			value <= (others=>'1');
			value(outwidth-1) <= '0';
		else
			value <= (others=>'0');
			value(outwidth-1) <= '1';
		end if;
		when "100" => -- sine
			value <= sineval & "00";
		when "101" => -- S & H
			value <= shval(outwidth-1 downto 0);
		when others =>
			value <= (others=>'-');
		end case;
	end if;
end process;
end implementation;

----------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
use work.types.all;
	
	-- invert output in this controller since value 0 is max amp, etc...
	
entity ampcalc is
	generic(
		sensewidth : integer := 12;
		envwidth : integer := 32;
		lfowidth : integer := 12;
		outwidth : integer := 9;
		baseampwidth : integer := 12
	);
	port(
		clk,rst : in std_logic;
		op_on : in std_logic;
		baseamp : in std_logic_vector(baseampwidth-1 downto 0);
		amodsense : in std_logic_vector(sensewidth-1 downto 0);
		envvalue : in std_logic_vector(envwidth-1 downto 0);
		lfovalue : in std_logic_vector(lfowidth-1 downto 0);
		outvalue : out std_logic_vector(outwidth-1 downto 0)
	);
end ampcalc;

architecture implementation of ampcalc is
	signal out_next : std_logic_vector(outwidth-1 downto 0);
	signal ulfovalue : std_logic_vector(lfowidth-1 downto 0);
begin
	ulfovalue <= (not lfovalue(lfowidth-1)) & lfovalue(lfowidth-2 downto 0);
comb : process(baseamp,op_on,amodsense,envvalue,ulfovalue)
	variable awitheg : std_logic_vector((baseampwidth*2)-1 downto 0);
	variable areduction : std_logic_vector(sensewidth+lfowidth-1 downto 0);
	variable reduceda : std_logic_vector((outwidth*2)-1 downto 0);
begin
	if(op_on = '1') then
		--areduction = amodsense * lfovalue;
		areduction := std_logic_vector(to_unsigned(to_integer(unsigned(amodsense)) * to_integer(unsigned(not ulfovalue)),sensewidth+lfowidth));
		awitheg := std_logic_vector(to_unsigned(to_integer(unsigned(envvalue(envwidth-1 downto envwidth-baseampwidth))) * to_integer(unsigned(baseamp)),baseampwidth*2));
		reduceda := std_logic_vector(to_unsigned(to_integer(unsigned(not areduction(sensewidth+lfowidth-1 downto sensewidth+lfowidth-outwidth))) * to_integer(unsigned(awitheg((baseampwidth*2)-1 downto (baseampwidth*2)-outwidth))),2*outwidth));
		out_next <= reduceda((outwidth*2)-1 downto outwidth);

	else
		out_next <= (others=>'0');
	end if;
end process;

sync : process(clk)
begin
	if(clk'event and clk = '1') then
		if(rst = '1') then
			outvalue <= (others=>'0');
		else
			outvalue <= not out_next; -- indexgenerator is inverted
		end if;
	end if;	
end process;
end implementation;


----------------------------------------

  -- Frequency calculation is exponential. To calculate any value between 2^-8 and 2^(7+255/256) in steps of 2^(1/256), a table is used for values 
  -- 2^7 to 2^(7+255/256) and the result is shifted between 0 and 15 times, adding -1  to exponent for each shift.
  
  -- This makes modulation very simple, since it's just adding env-value and (lfovalue*pmodsense) to get a exponential which is 
  -- multiplied with the base frequency to get the final frequency sent to OPS.
  
  -- pipelined idea
  -- step 1: generate lookup value (pshiftcalc)
  -- step 2: multiply freq with lookup value (freqcalc)

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
use work.types.all;
use work.egs_package.all;

entity pshiftcalc is
	port(
		clk,rst : in std_logic;
		pmodsense : in std_logic_vector(PMODWIDTH-1 downto 0);
		envvalue : in std_logic_vector(EGCOUNTWIDTH-1 downto 0);
		lfovalue : in std_logic_vector(LFOWIDTH-1 downto 0);
		multconst : out std_logic_vector(23 downto 0);
		shamt : out std_logic_vector(3 downto 0)
	);
end pshiftcalc;

architecture implementation of pshiftcalc is
	signal shamt_next : std_logic_vector(3 downto 0);
	signal multconst_next : std_logic_vector(23 downto 0);
begin

comb : process(pmodsense, envvalue, lfovalue)
	variable lforesult : std_logic_vector(LFOWIDTH+PMODWIDTH-1 downto 0);
	variable pmult : std_logic_vector(11 downto 0);
	variable signedeg : std_logic_vector(10 downto 0);
begin
	signedeg := (not envvalue(EGCOUNTWIDTH-1)) & envvalue(EGCOUNTWIDTH-2 downto EGCOUNTWIDTH-11);
	lforesult := std_logic_vector(to_signed(to_integer(unsigned(pmodsense)) * to_integer(signed(lfovalue)),LFOWIDTH+PMODWIDTH));
	pmult := std_logic_vector(to_signed(to_integer(signed(signedeg)) + to_integer(signed(lforesult(LFOWIDTH+PMODWIDTH-1 downto LFOWIDTH+PMODWIDTH-9))),12));
	shamt_next <= pmult(11) & (not pmult(10 downto 8));
	multconst_next <= exp_table(to_integer(unsigned(pmult(7 downto 0))));
end process;


sync : process(clk)

begin
	if(clk'event and clk = '1') then
		if(rst = '1') then
			multconst <= x"010000";
			shamt <= "0000";
		else
			multconst <= multconst_next;
			shamt <= shamt_next;
		end if;
	end if;
end process;

end implementation;

----------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;
use work.types.all;
use work.egs_package.all;

entity freqcalc is
	port(
		clk,rst : in std_logic;
		basicfreq : in std_logic_vector(19 downto 0);
		multconst : in std_logic_vector(23 downto 0);
		shamt : in std_logic_vector(3 downto 0);
		outvalue : out std_logic_vector(19 downto 0)
	);
end freqcalc;
  
architecture implementation of freqcalc is
	signal out_next : std_logic_vector(19 downto 0);
	signal tableindex : std_logic_vector(7 downto 0); -- table lookup index
	signal shiftedconst : std_logic_vector(23 downto 0);
	signal multresult : std_logic_vector(27 downto 0);
	signal fracresult : std_logic_vector(31 downto 0);
	signal intresult : std_logic_vector(27 downto 0);
begin
shiftedconst <= std_logic_vector(shift_right(unsigned(multconst),to_integer(unsigned(shamt))));
fracresult <= std_logic_vector(to_unsigned(to_integer(unsigned(basicfreq)) * to_integer(unsigned(shiftedconst(15 downto 4))),32));
intresult <= std_logic_vector(to_unsigned(to_integer(unsigned(basicfreq)) * to_integer(unsigned(shiftedconst(23 downto 16))),28));
multresult <= std_logic_vector(unsigned(fracresult(31 downto 12)) + unsigned(intresult));
out_next <= multresult(19 downto 0);

sync : process(clk)
	
begin
	if(clk'event and clk = '1') then
		if(rst = '1') then
			outvalue <= (others=>'0');
		else
			outvalue <= out_next;
		end if;
	end if;	
end process;
end implementation;

