
library IEEE;
use IEEE.STD_LOGIC_1164.all;

package egs_package is
	constant EGS_GL_STATEWIDTH : integer := 91;
	constant EGS_OP_STATEWIDTH : integer := 34;
	constant EGS_PAR0WIDTH		: integer := 12;
	constant EGS_PAR1WIDTH		: integer := 12;
	constant EGS_PAR2WIDTH		: integer := 12;
	constant EGS_PAR3WIDTH		: integer := 12;
	constant EGS_PAR4WIDTH		: integer := 20;
	constant EGS_PAR5WIDTH		: integer := 12;
	constant EGS_PAR6WIDTH		: integer := 14;
	constant EGS_PAR7WIDTH		: integer := 17;
	
	constant EGS_EGRATEWIDTH	: integer := 22;
	constant EGS_RAMWIDTH		: integer := 216;

	constant FSL_RATEBITOFFSET : integer := 24;
	constant FSL_STAGEOFFSET	: integer := 22;
	constant FSL_PAROFFSET		: integer := 21;
	constant FSL_CHOFFSET		: integer := 28;
	constant FSL_OPOFFSET		: integer := 25;
	constant PARBITS				: integer := 3;
	
	constant OPBITS				: integer := 3;
	constant CHBITS				: integer := 4;
	
	constant LFOWAVEWIDTH : integer := 3;
	constant LFOPHASEWIDTH : integer := 20;
	constant LFOSPEEDWIDTH : integer := 12;
	constant LFODELAYWIDTH : integer := 20;
	constant LFOWIDTH : integer := 12;

	constant TLWIDTH : integer := 9;

	constant AMODWIDTH : integer := 12;
	constant PMODWIDTH : integer := 12;

	constant EGSTAGEWIDTH : integer := 2;
	constant EGCOUNTWIDTH : integer := 32;
	constant EGRWIDTH : integer := EGS_EGRATEWIDTH;
	constant EGLWIDTH : integer := 12;
	constant EGSBITS : integer := 4;
	
	constant OPAMPWIDTH : integer := 12;
	constant OPFREQWIDTH : integer := 20;

	constant GLR_LFODELAYCOUNTOFFSET : integer := 0;
	constant GLR_PMODOFFSET : integer := GLR_LFODELAYCOUNTOFFSET + LFODELAYWIDTH;
	constant GLR_KEYONOFFSET : integer := GLR_PMODOFFSET + PMODWIDTH;
	constant GLR_LFOWAVEOFFSET : integer := GLR_KEYONOFFSET + 1;
	constant GLR_LFOPHASEOFFSET : integer := GLR_LFOWAVEOFFSET + LFOWAVEWIDTH;
	constant GLR_LFOSTATEOFFSET : integer := GLR_LFOPHASEOFFSET + LFOPHASEWIDTH;
	constant GLR_EGCOUNTOFFSET : integer := GLR_LFOSTATEOFFSET + 1;
	constant GLR_EGSTAGEOFFSET : integer := GLR_EGCOUNTOFFSET + EGCOUNTWIDTH;

	constant PHASEWIDTH : integer := 20;
   constant AMPWIDTH : integer := 16;

	constant OP_EGCOUNTOFFSET : integer := 0;
	constant OP_EGSTAGEOFFSET : integer := OP_EGCOUNTOFFSET + EGCOUNTWIDTH;
	constant OP_ONOFFSET : integer := OP_EGSTAGEOFFSET + EGSTAGEWIDTH;
	constant OP_SYNCONOFFSET : integer := OP_ONOFFSET + 1;
	constant OP_OUTLEVELOFFSET : integer := OP_SYNCONOFFSET + 1;
	constant OP_AMODOFFSET : integer := OP_OUTLEVELOFFSET + OPAMPWIDTH;
	constant OP_FREQOFFSET : integer := OP_AMODOFFSET + AMODWIDTH;
	constant OP_L4OFFSET : integer := OP_FREQOFFSET + OPFREQWIDTH;
	constant OP_L3OFFSET : integer := OP_L4OFFSET + EGLWIDTH;
	constant OP_L2OFFSET : integer := OP_L3OFFSET + EGLWIDTH;
	constant OP_L1OFFSET : integer := OP_L2OFFSET + EGLWIDTH;
	constant OP_R4OFFSET : integer := OP_L1OFFSET + EGLWIDTH;
	constant OP_R3OFFSET : integer := OP_R4OFFSET + EGRWIDTH;
	constant OP_R2OFFSET : integer := OP_R3OFFSET + EGRWIDTH;
	constant OP_R1OFFSET : integer := OP_R2OFFSET + EGRWIDTH;	
	
	constant GL_KEYONOFFSET : integer := OP_AMODOFFSET-EGS_PAR7WIDTH;
	constant GL_LFOWAVEOFFSET : integer := GL_KEYONOFFSET + 1;
	constant GL_LFOSYNCOFFSET : integer := GL_LFOWAVEOFFSET + LFOWAVEWIDTH;
	constant GL_LFOSPEEDOFFSET : integer := GL_LFOSYNCOFFSET + 1;
	
	constant GL_PMODOFFSET : integer := OP_AMODOFFSET;
	constant GL_LFODELAYOFFSET : integer := GL_PMODOFFSET + PMODWIDTH;
	
	constant EGS_PAR0OFFSET		: integer := OP_L1OFFSET;
	constant EGS_PAR1OFFSET		: integer := OP_L2OFFSET;
	constant EGS_PAR2OFFSET		: integer := OP_L3OFFSET;
	constant EGS_PAR3OFFSET		: integer := OP_L4OFFSET;
	constant EGS_PAR4OFFSET		: integer := OP_FREQOFFSET;
	constant EGS_PAR5OFFSET		: integer := OP_AMODOFFSET;
	constant EGS_PAR6OFFSET		: integer := OP_ONOFFSET;
	constant EGS_PAR7OFFSET		: integer := EGS_PAR5OFFSET-EGS_PAR7WIDTH;
	
	type exptype is array (0 to 255) of std_logic_vector(23 downto 0);
	constant exp_table : exptype :=
				(  x"800000",x"8058d8",x"80b1ed",x"810b41",x"8164d2",x"81bea1",x"8218af",x"8272fc",
					x"82cd87",x"832850",x"838359",x"83dea1",x"843a29",x"8495f0",x"84f1f6",x"854e3d",
					x"85aac3",x"86078a",x"866491",x"86c1d9",x"871f62",x"877d2b",x"87db35",x"883981",
					x"88980f",x"88f6dd",x"8955ee",x"89b541",x"8a14d5",x"8a74ad",x"8ad4c6",x"8b3523",
					x"8b95c2",x"8bf6a4",x"8c57ca",x"8cb933",x"8d1adf",x"8d7cd0",x"8ddf04",x"8e417d",
					x"8ea43a",x"8f073b",x"8f6a81",x"8fce0c",x"9031dc",x"9095f2",x"90fa4d",x"915eed",
					x"91c3d3",x"922900",x"928e72",x"92f42c",x"935a2b",x"93c072",x"9426ff",x"948dd4",
					x"94f4f0",x"955c53",x"95c3ff",x"962bf2",x"96942d",x"96fcb1",x"97657d",x"97ce92",
					x"9837f0",x"98a197",x"990b88",x"9975c2",x"99e046",x"9a4b13",x"9ab62b",x"9b218d",
					x"9b8d3a",x"9bf931",x"9c6573",x"9cd201",x"9d3eda",x"9dabfe",x"9e196e",x"9e872a",
					x"9ef532",x"9f6387",x"9fd228",x"a04116",x"a0b051",x"a11fd9",x"a18faf",x"a1ffd2",
					x"a27043",x"a2e102",x"a3520f",x"a3c36b",x"a43516",x"a4a70f",x"a51958",x"a58bef",
					x"a5fed7",x"a6720e",x"a6e595",x"a7596c",x"a7cd94",x"a8420c",x"a8b6d5",x"a92bef",
					x"a9a15b",x"aa1718",x"aa8d26",x"ab0387",x"ab7a3a",x"abf13f",x"ac6897",x"ace041",
					x"ad583f",x"add090",x"ae4934",x"aec22d",x"af3b79",x"afb519",x"b02f0e",x"b0a957",
					x"b123f6",x"b19ee9",x"b21a32",x"b295d0",x"b311c4",x"b38e0e",x"b40aaf",x"b487a6",
					x"b504f3",x"b58298",x"b60094",x"b67ee7",x"b6fd92",x"b77c95",x"b7fbf0",x"b87ba3",
					x"b8fbaf",x"b97c14",x"b9fcd2",x"ba7dea",x"baff5b",x"bb8126",x"bc034a",x"bc85ca",
					x"bd08a4",x"bd8bd8",x"be0f68",x"be9353",x"bf179a",x"bf9c3c",x"c0213b",x"c0a695",
					x"c12c4d",x"c1b261",x"c238d2",x"c2bfa1",x"c346cd",x"c3ce57",x"c4563f",x"c4de85",
					x"c5672a",x"c5f02e",x"c67991",x"c70353",x"c78d75",x"c817f7",x"c8a2d8",x"c92e1b",
					x"c9b9be",x"ca45c1",x"cad226",x"cb5eed",x"cbec15",x"cc799f",x"cd078c",x"cd95da",
					x"ce248c",x"ceb3a1",x"cf4319",x"cfd2f4",x"d06334",x"d0f3d7",x"d184df",x"d2164c",
					x"d2a81e",x"d33a54",x"d3ccf1",x"d45ff3",x"d4f35b",x"d58729",x"d61b5e",x"d6affa",
					x"d744fd",x"d7da67",x"d87039",x"d90673",x"d99d16",x"da3421",x"dacb94",x"db6371",
					x"dbfbb8",x"dc9468",x"dd2d82",x"ddc706",x"de60f5",x"defb4e",x"df9613",x"e03143",
					x"e0ccdf",x"e168e7",x"e2055b",x"e2a23c",x"e33f89",x"e3dd44",x"e47b6d",x"e51a03",
					x"e5b907",x"e65879",x"e6f85b",x"e798ab",x"e8396a",x"e8da99",x"e97c38",x"ea1e47",
					x"eac0c7",x"eb63b7",x"ec0719",x"ecaaec",x"ed4f30",x"edf3e7",x"ee9910",x"ef3eab",
					x"efe4ba",x"f08b3b",x"f13231",x"f1d99a",x"f28177",x"f329c9",x"f3d290",x"f47bcc",
					x"f5257d",x"f5cfa4",x"f67a41",x"f72555",x"f7d0df",x"f87ce1",x"f9295a",x"f9d64a",
					x"fa83b3",x"fb3194",x"fbdfed",x"fc8ec0",x"fd3e0c",x"fdedd2",x"fe9e11",x"ff4ecb"
					);

		
component egsram
	port (
	clka: IN std_logic;
	wea: IN std_logic_VECTOR(0 downto 0);
	addra: IN std_logic_VECTOR(6 downto 0);
	dina: IN std_logic_VECTOR(215 downto 0);
	clkb: IN std_logic;
	addrb: IN std_logic_VECTOR(6 downto 0);
	doutb: OUT std_logic_VECTOR(215 downto 0));
end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of egsram: component is true;
 
 
component globram
	port (
	clka: IN std_logic;
	wea: IN std_logic_VECTOR(0 downto 0);
	addra: IN std_logic_VECTOR(3 downto 0);
	dina: IN std_logic_VECTOR(90 downto 0);
	clkb: IN std_logic;
	addrb: IN std_logic_VECTOR(3 downto 0);
	doutb: OUT std_logic_VECTOR(90 downto 0));
end component;

-- Synplicity black box declaration
attribute syn_black_box of globram: component is true;
	
component indexgen is
	generic(
		width : integer := 14
	);
	port(
		tlval : in std_logic_vector(8 downto 0);
		output : out std_logic_vector(width-1 downto 0)
	);
end component;

component lfo_phaseacc is
	generic(
		regwidth : integer := 20;
		incwidth : integer := 12;
		delaywidth : integer := 20
	);
	port(
		clk,rst : in std_logic;
		pinc : in std_logic_vector(incwidth-1 downto 0);
		sync_on : in std_logic;
		new_key : in std_logic;
		delay : in std_logic_vector(delaywidth-1 downto 0); -- given in cycles
		dstate_in : in std_logic;
		delayreg_in : in std_logic_vector(delaywidth-1 downto 0);
		phasereg_in : in std_logic_vector(regwidth-1 downto 0);
		dstate_out : out std_logic;
		delayreg_out : out std_logic_vector(delaywidth-1 downto 0);
		phasereg_out : out std_logic_vector(regwidth-1 downto 0)
	);
end component;

component lfo_wavegen is
	generic(
		phasewidth : integer := 20;
		outwidth : integer := 12
	);
	port(
		phase : in std_logic_vector(phasewidth-1 downto 0);
		wave : in std_logic_vector(2 downto 0);
		shval : in std_logic_vector(phasewidth-1 downto 0);
		sineval : in std_logic_vector(9 downto 0);
		dstate : in std_logic;
		value : out std_logic_vector(outwidth-1 downto 0)
	);
end component;

component ampcalc is
	generic(
		sensewidth : integer := 12;
		envwidth : integer := 32;
		lfowidth : integer := 12;
		outwidth : integer := 9;
		baseampwidth : integer := 12
	);
	port(
		clk,rst : in std_logic;
		op_on : in std_logic;
		baseamp : in std_logic_vector(baseampwidth-1 downto 0);
		amodsense : in std_logic_vector(sensewidth-1 downto 0);
		envvalue : in std_logic_vector(envwidth-1 downto 0);
		lfovalue : in std_logic_vector(lfowidth-1 downto 0);
		outvalue : out std_logic_vector(outwidth-1 downto 0)
	);
end component;

component pshiftcalc is
	port(
		clk,rst : in std_logic;
		pmodsense : in std_logic_vector(PMODWIDTH-1 downto 0);
		envvalue : in std_logic_vector(EGCOUNTWIDTH-1 downto 0);
		lfovalue : in std_logic_vector(LFOWIDTH-1 downto 0);
		multconst : out std_logic_vector(23 downto 0);
		shamt : out std_logic_vector(3 downto 0)
	);
end component;

component freqcalc is
	port(
		clk,rst : in std_logic;
		basicfreq : in std_logic_vector(19 downto 0);
		multconst : in std_logic_vector(23 downto 0);
		shamt : in std_logic_vector(3 downto 0);
		outvalue : out std_logic_vector(19 downto 0)
	);
end component;

component rate_converter is
	generic(
		INWIDTH	: integer := 22;
		OUTWIDTH : integer := 33;
		SBITS		: integer := 4
	);
	port(
		clk,rst : in std_logic;
		r1_in : in std_logic_vector(INWIDTH-1 downto 0);
		r2_in : in std_logic_vector(INWIDTH-1 downto 0);
		r3_in : in std_logic_vector(INWIDTH-1 downto 0);
		r4_in : in std_logic_vector(INWIDTH-1 downto 0);
		r1_out : out std_logic_vector(OUTWIDTH-1 downto 0);
		r2_out : out std_logic_vector(OUTWIDTH-1 downto 0);
		r3_out : out std_logic_vector(OUTWIDTH-1 downto 0);
		r4_out : out std_logic_vector(OUTWIDTH-1 downto 0)
	);
end component rate_converter;

component eg is
	generic(
		LWIDTH	: integer := 12;
		EGWIDTH	: integer := 32
	);
	port(
		r1_in : in std_logic_vector(EGWIDTH downto 0);
		r2_in : in std_logic_vector(EGWIDTH downto 0);
		r3_in : in std_logic_vector(EGWIDTH downto 0);
		r4_in : in std_logic_vector(EGWIDTH downto 0);
		l1_in : in std_logic_vector(LWIDTH-1 downto 0);
		l2_in : in std_logic_vector(LWIDTH-1 downto 0);
		l3_in : in std_logic_vector(LWIDTH-1 downto 0);
		l4_in : in std_logic_vector(LWIDTH-1 downto 0);
		cur_stage : in std_logic_vector(1 downto 0);
		cur_count : in std_logic_vector(EGWIDTH-1 downto 0);
		key_on	: in std_logic;
		next_stage : out std_logic_vector(1 downto 0);
		next_count : out std_logic_vector(EGWIDTH-1 downto 0)
	);
end component;

component sine_lut
		port (
		clk: IN std_logic;
		phase_in: IN std_logic_VECTOR(15 downto 0);
		sine: OUT std_logic_VECTOR(9 downto 0));
end component;

end egs_package;