-------------------------------------------------------------------------------
-- Filename:        fsl_hwa.vhd
-- Version:         v1.00a
-- Description:     Fast Simplex Link core
--
-------------------------------------------------------------------------------
-- Structure: 
--              fsl_hwa.vhd
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;

--  Add user library if this source file uses
--  entities from the user library
library fsl_hwa_v1_00_a; --USER-- library name
use fsl_hwa_v1_00_a.all; --USER-- use statement

-------------------------------------------------------------------------------
-- Entity Section
-------------------------------------------------------------------------------

entity fsl_hwa is 
  generic (
	C_FSL_DWIDTH	: integer := 32; -- fsl data size
	C_EXT_RESET_HIGH  : integer := 1 -- external reset is active high
  ); 
  port
  (
    --Required FSL ports, do not add to or delete
    FSL_Clk	     : in std_logic; -- fsl clock
    FSL_Rst	     : in std_logic; -- reset signal for the core

    -- the slave part of the FSL interface
    FSL_S_Clk    : out std_logic; -- slave clock, the same as FSL_Clk
    FSL_S_Read   : out std_logic; -- read ack from the core
    FSL_S_Data   : in std_logic_vector(0 to C_FSL_DWIDTH-1); -- data to the core
    FSL_S_Control: in std_logic; -- active if control info is passed over
    FSL_S_Exists : in std_logic; -- active if data is available

    -- the master part of the FSL interface
    FSL_M_Clk    : out std_logic; -- slave clock, the same as FSL_Clk
    FSL_M_Write  : out std_logic; -- read ack from the core
    FSL_M_Data   : out std_logic_vector(0 to C_FSL_DWIDTH-1); -- data to the core
    FSL_M_Control: out std_logic; -- active if control info is passed over
    FSL_M_Full   : in std_logic; -- active if fifo is full

    -- i2s signals
    I2S_MCLK     : out std_logic;
    I2S_LRCK     : out std_logic;
    I2S_SCLK     : out std_logic;
    I2S_DOUT     : out std_logic;
	 DAC_MCLK     : in std_logic
  );

      --fan-out attributes for XST
 attribute MAX_FANOUT                    : string;
 attribute MAX_FANOUT   of FSL_Clk       : signal is "10000";
 attribute MAX_FANOUT   of FSL_Rst       : signal is "10000";
 attribute MAX_FANOUT   of FSL_S_Clk     : signal is "10000";
 attribute MAX_FANOUT   of FSL_M_Clk     : signal is "10000";

end entity fsl_hwa; --USER-- change entity name

-------------------------------------------------------------------------------
-- Architecture Section
-------------------------------------------------------------------------------

architecture imp of fsl_hwa is --USER-- change entity name

signal FSL_Rst_ni: std_logic;

component dx7_hw_top
	port(		
		clk      : in std_logic; -- clock
		rst_n    : in std_logic; -- reset, active low
		-- read signals
		exists   : in std_logic; -- active if data is available
		rd_ack   : out std_logic; -- read ack from the core
		d_in     : in std_logic_vector(0 to 31); -- data to the core
		-- write signals
		full     : in std_logic; -- active if fifo is full
		wr_en    : out std_logic; -- read ack from the core
		d_out    : out std_logic_vector(0 to 31); -- data to the core
		-- i2s signals
		i2s_mclk     : out std_logic;
		i2s_lrck     : out std_logic;
		i2s_sclk     : out std_logic;
		i2s_dout     : out std_logic;
		DAC_MCLK     : in std_logic
	);
end component;

------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------

dx7_inst : dx7_hw_top
  port map
  (
    Clk => FSL_Clk,
    Rst_n => FSL_Rst_ni,
    Exists => FSL_S_Exists,
    Rd_ack => FSL_S_Read,
    D_in => FSL_S_Data,
    Full => FSL_M_Full,
    Wr_en => FSL_M_Write,
    D_out => FSL_M_Data,
    i2s_mclk => I2S_MCLK,
    i2s_lrck => I2S_LRCK, 
    i2s_sclk => I2S_SCLK, 
    i2s_dout => I2S_DOUT,
	 DAC_MCLK => DAC_MCLK
 );
  
FSL_S_Clk   <= FSL_Clk; 
FSL_M_Clk   <= FSL_Clk;
FSL_M_Control <= '0';

-- reset signal, active low
FSL_Rst_ni <= not FSL_Rst when C_EXT_RESET_HIGH = 1 else FSL_Rst;

end architecture imp;

