----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:28:07 10/23/2011 
-- Design Name: 
-- Module Name:    gi2s - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.types.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity i2sctrl is

	generic (
		MBITS : integer := 10; -- Width for mclk counter
		SBITS : integer := 4; -- width for sclk counter
		DBITS : integer := 24; -- Audio Data bits
		MLRCK : integer := 768; -- mclk / lrck ratio
		MSCLK : integer := 16 -- mclk / sclk ratio
	);
    Port ( clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           data_l : in  STD_LOGIC_VECTOR (DBITS-1 downto 0);
           data_r : in  STD_LOGIC_VECTOR (DBITS-1 downto 0);
           mclk_out : out  STD_LOGIC;
           lrck_out : out  STD_LOGIC;
           dac_out : out  STD_LOGIC;
           sclk_out : out  STD_LOGIC);
end i2sctrl;

architecture Behavioral of i2sctrl is
	type i2sreg is
	record
		lcount			: std_logic_vector(MBITS-1 downto 0); -- counter to store lrck pos
		scount			: std_logic_vector(SBITS-1 downto 0); -- counter to store sclk pos
		dcount			: std_logic_vector(4 downto 0); -- counter to store sample pos
		channel				: std_logic;
		schannel				: std_logic;
		DACout				: std_logic;
		sclkout				: std_logic;
		load_sample			: std_logic;
		iload_sample		: std_logic;
	end record;
	signal r, r_next : i2sreg;
	signal lsample,rsample : std_logic_vector(DBITS-1 downto 0);
	signal ilsample,irsample : std_logic_vector(DBITS-1 downto 0);
begin
lreg : data_reg generic map(24) port map(clk,rst,ilsample,lsample,r.load_sample);
rreg : data_reg generic map(24) port map(clk,rst,irsample,rsample,r.load_sample);
ilreg : data_reg generic map(24) port map(clk,rst,data_l,ilsample,r.iload_sample);
irreg : data_reg generic map(24) port map(clk,rst,data_r,irsample,r.iload_sample);

mclk_out <= clk;
dac_out <= r.DACout;

lrdelay : bit_dff port map(clk,rst,r.channel,lrck_out);
sdelay : bit_dff port map(clk,rst,r.sclkout,sclk_out);

comb : process(r,lsample,rsample,rst)
	variable v : i2sreg;
begin
	v := r;
	v.iload_sample := '0';
	v.load_sample := '0';
	
	-- update counter for lrclock. Load new sample on h to l transition
	if(r.lcount = (MLRCK/2)-1) then
		v.lcount := (others=>'0');
		v.channel := not r.channel;
		if(r.channel = '1') then
			v.iload_sample := '1';
		end if;
	else
		v.lcount := r.lcount + 1;
	end if;
	
	-- udpate counter for sclock
	if(r.scount = (MSCLK/2)-1) then -- 
		v.scount := (others=>'0');
		v.sclkout := not r.sclkout;
		-- update dcount on neg transition. Remember: Data is valid on rising sclk edge
		if(r.sclkout = '1') then
			if (r.dcount = 0) then -- last bit sent, switch channel
				v.dcount := conv_std_logic_vector(DBITS-1,5);
				v.schannel := not r.schannel;
				if(r.schannel = '1') then
					v.load_sample := '1';
				end if;
			else
				v.dcount := r.dcount - 1;
			end if;
		end if;
	else
		v.scount := r.scount + 1;
	end if;
	
	-- get data from register
	if(r.schannel = '0') then -- left
		v.DACout := lsample(conv_integer(unsigned(r.dcount)));
	else
		v.DACout := rsample(conv_integer(unsigned(r.dcount)));
	end if;
	
	if(rst = '1') then
		v.lcount := (others=>'0');
		v.scount := (others=>'0');
		v.dcount := (others=>'0');
		v.channel := '0';
		v.schannel := '1';
		v.DACout := '0';
		v.load_sample := '0';
		v.iload_sample := '0';
		v.sclkout := '0';
	end if;
	r_next <= v;
end process;
	
sync : process (clk)
begin
	if(clk'event and clk = '1') then
		r <= r_next;
	end if;
end process;

	
end Behavioral;

