----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:40:18 09/13/2011 
-- Design Name: 
-- Module Name:    mixer - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use work.types.all;

entity mixer is
	generic(
		IN_BITS : integer := 16;
		OUT_BITS : integer := 16;
		CTRL_BITS : integer range 1 to 16 := 8
	);
    Port ( clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           a_in : in  STD_LOGIC_VECTOR (IN_BITS-1 downto 0);
           l_out : out  STD_LOGIC_VECTOR (OUT_BITS-1 downto 0);
			  r_out : out  STD_LOGIC_VECTOR (OUT_BITS-1 downto 0);
           vol : in  STD_LOGIC_VECTOR (CTRL_BITS-1 downto 0);
           pan : in  STD_LOGIC_VECTOR (CTRL_BITS-1 downto 0));
end mixer;

architecture Behavioral of mixer is
	type mix_rec is record
		left : std_logic_vector(2*CTRL_BITS+IN_BITS-1 downto 0);
		right : std_logic_vector(2*CTRL_BITS+IN_BITS-1 downto 0);
	end record;
	signal r, r_next : mix_rec;

begin

l_out <= r.left((IN_BITS+2*CTRL_BITS)-6 downto IN_BITS+(2*CTRL_BITS)-OUT_BITS-5);
r_out <= r.right((IN_BITS+2*CTRL_BITS)-6 downto IN_BITS+(2*CTRL_BITS)-OUT_BITS-5);

comb : process(r, a_in, rst, vol, pan)
	variable v : mix_rec;
	variable audio : integer; -- is CTRLBITS+IN_BITS wide
	variable leftpan, rightpan : std_logic_vector(CTRL_BITS-1 downto 0);
begin
	v := r;
	leftpan := pan;
	rightpan := not pan;
	audio := to_integer(signed(a_in)) * to_integer(unsigned(vol));
	
	-- out of multipliers. Removing pan unfortunately
	--v.left := std_logic_vector(to_signed(audio * to_integer(unsigned(leftpan)), 2*CTRL_BITS+IN_BITS));
	--v.right := std_logic_vector(to_signed(audio * to_integer(unsigned(rightpan)), 2*CTRL_BITS+IN_BITS));
	v.left := std_logic_vector(to_signed(audio,CTRL_BITS+IN_BITS)) & x"00";
	v.right := std_logic_vector(to_signed(audio,CTRL_BITS+IN_BITS)) & x"00";
	
	if(rst = '1') then
		v.left := (others=>'0');
		v.right := (others=>'0');
	end if;
	r_next <= v;
end process;

sync : process(clk)
begin
	if(clk'event and clk = '1') then
		r <= r_next;
	end if;
end process;

end Behavioral;

